@@ -157,8 +157,10 @@ static bool ignore_warnings = false;
 static bool fast_scan = false;
 static bool negate = false;
 static bool print_count_only = false;
+static bool strict_escape = false;
 static bool fail_on_warnings = false;
 static bool rules_are_compiled = false;
+static bool disable_console_logs = false;
 static long total_count = 0;
 static long limit = 0;
 static long timeout = 1000000;
@@ -191,6 +193,12 @@ args_option_t options[] = {
         &print_count_only,
         _T("print only number of matches")),
 
+    OPT_BOOLEAN(
+        'E',
+        _T("strict-escape"),
+        &strict_escape,
+        _T("warn on unknown escape sequences")),
+
     OPT_STRING_MULTI(
         'd',
         _T("define"),
@@ -199,6 +207,12 @@ args_option_t options[] = {
         _T("define external variable"),
         _T("VAR=VALUE")),
 
+    OPT_BOOLEAN(
+        'q',
+        _T("disable-console-logs"),
+        &disable_console_logs,
+        _T("disable printing console log messages")),
+
     OPT_BOOLEAN(
         0,
         _T("fail-on-warnings"),
@@ -674,14 +688,15 @@ static int scan_dir(const char* dir, SCAN_OPTIONS* scan_opts)
         de = readdir(dp);
         continue;
       }
-      // If the directory entry is a symlink, check if it points to . and
-      // skip it in that case.
+      // If the directory entry is a symlink, check if it points to . or .. and
+      // skip it in those cases.
       else if (S_ISLNK(st.st_mode))
       {
         char buf[2];
         int len = readlink(full_path, buf, sizeof(buf));
 
-        if (len == 1 && buf[0] == '.')
+        if ((len == 1 && buf[0] == '.') ||
+            (len == 2 && buf[0] == '.' && buf[1] == '.'))
         {
           de = readdir(dp);
           continue;
@@ -1225,6 +1240,28 @@ static int callback(
 
     return CALLBACK_CONTINUE;
 
+  case CALLBACK_MSG_TOO_SLOW_SCANNING:
+    if (ignore_warnings)
+      return CALLBACK_CONTINUE;
+
+    string = (YR_STRING*) message_data;
+    rule = &context->rules->rules_table[string->rule_idx];
+
+    if (rule != NULL && string != NULL)
+      fprintf(
+          stderr,
+          "warning: rule \"%s\": scanning with string %s is taking a very long "
+          "time, it is either too general or very common.\n",
+          rule->identifier,
+          string->identifier);
+    else
+      return CALLBACK_CONTINUE;
+
+    if (fail_on_warnings)
+      return CALLBACK_ERROR;
+
+    return CALLBACK_CONTINUE;
+
   case CALLBACK_MSG_TOO_MANY_MATCHES:
     if (ignore_warnings)
       return CALLBACK_CONTINUE;
@@ -1245,7 +1282,8 @@ static int callback(
     return CALLBACK_CONTINUE;
 
   case CALLBACK_MSG_CONSOLE_LOG:
-    _tprintf(_T("%" PF_S "\n"), (char*) message_data);
+    if (!disable_console_logs)
+      _tprintf(_T("%" PF_S "\n"), (char*) message_data);
     return CALLBACK_CONTINUE;
   }
 
@@ -1535,6 +1573,11 @@ int _tmain(int argc, const char_t** argv)
 
     yr_compiler_set_callback(compiler, print_compiler_error, &cr);
 
+    if (strict_escape)
+      compiler->strict_escape = true;
+    else
+      compiler->strict_escape = false;
+
     if (!compile_files(compiler, argc, argv))
       exit_with_code(EXIT_FAILURE);
 
