@@ -45,6 +45,7 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #include <yara/sizedstr.h>
 #include <yara/stopwatch.h>
 #include <yara/strutils.h>
+#include <yara/unaligned.h>
 #include <yara/utils.h>
 
 #define MEM_SIZE YR_MAX_LOOP_NESTING*(YR_MAX_LOOP_VARS + YR_INTERNAL_LOOP_VARS)
@@ -130,7 +131,7 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
           offset <= block->base + block->size - sizeof(type))     \
       {                                                           \
         type result;                                              \
-        const uint8_t* data = block->fetch_data(block);           \
+        const uint8_t* data = yr_fetch_block_data(block);         \
         if (data == NULL)                                         \
           return YR_UNDEFINED;                                    \
         result = *(type*) (data + offset - block->base);          \
@@ -157,15 +158,21 @@ function_read(int32_t, big_endian);
 
 static const uint8_t* jmp_if(int condition, const uint8_t* ip)
 {
-  size_t off;
+  int32_t off = 0;
 
   if (condition)
   {
-    // The condition is true, the instruction pointer is incremented in the
-    // amount specified by the jump's offset. The offset is relative to the
-    // jump opcode, but now the instruction pointer is pointing past the opcode
-    // that's why we decrement the offset by 1.
-    off = *(int32_t*) (ip) -1;
+    // The condition is true, the instruction pointer (ip) is incremented in
+    // the amount specified by the jump's offset, which is a int32_t following
+    // the jump opcode. The ip is currently past the opcode and pointing to
+    // the offset.
+
+    // Copy the offset from the instruction stream to a local variable.
+    off = yr_unaligned_u32(ip);
+
+    // The offset is relative to the jump opcode, but now the ip is one byte
+    // past the opcode, so we need to decrement it by one.
+    off -= 1;
   }
   else
   {
@@ -326,6 +333,68 @@ static int iter_int_enum_next(YR_ITERATOR* self, YR_VALUE_STACK* stack)
   return ERROR_SUCCESS;
 }
 
+static int iter_string_set_next(YR_ITERATOR* self, YR_VALUE_STACK* stack)
+{
+  // Check that there's two available slots in the stack, one for the next
+  // item returned by the iterator and another one for the boolean that
+  // indicates if there are more items.
+  if (stack->sp + 1 >= stack->capacity)
+    return ERROR_EXEC_STACK_OVERFLOW;
+
+  // If the current index is equal or larger than array's length the iterator
+  // has reached the end of the array.
+  if (self->string_set_it.index >= self->string_set_it.count)
+    goto _stop_iter;
+
+  // Push the false value that indicates that the iterator is not exhausted.
+  stack->items[stack->sp++].i = 0;
+  stack->items[stack->sp++].s =
+      self->string_set_it.strings[self->string_set_it.index];
+  self->string_set_it.index++;
+
+  return ERROR_SUCCESS;
+
+_stop_iter:
+
+  // Push true for indicating the iterator has been exhausted.
+  stack->items[stack->sp++].i = 1;
+  // Push YR_UNDEFINED as a placeholder for the next item.
+  stack->items[stack->sp++].i = YR_UNDEFINED;
+
+  return ERROR_SUCCESS;
+}
+
+static int iter_text_string_set_next(YR_ITERATOR* self, YR_VALUE_STACK* stack)
+{
+  // Check that there's two available slots in the stack, one for the next
+  // item returned by the iterator and another one for the boolean that
+  // indicates if there are more items.
+  if (stack->sp + 1 >= stack->capacity)
+    return ERROR_EXEC_STACK_OVERFLOW;
+
+  // If the current index is equal or larger than array's length the iterator
+  // has reached the end of the array.
+  if (self->text_string_set_it.index >= self->text_string_set_it.count)
+    goto _stop_iter;
+
+  // Push the false value that indicates that the iterator is not exhausted.
+  stack->items[stack->sp++].i = 0;
+  stack->items[stack->sp++].ss =
+      self->text_string_set_it.strings[self->text_string_set_it.index];
+  self->text_string_set_it.index++;
+
+  return ERROR_SUCCESS;
+
+_stop_iter:
+
+  // Push true for indicating the iterator has been exhausted.
+  stack->items[stack->sp++].i = 1;
+  // Push YR_UNDEFINED as a placeholder for the next item.
+  stack->items[stack->sp++].i = YR_UNDEFINED;
+
+  return ERROR_SUCCESS;
+}
+
 // Global table that contains the "next" function for different types of
 // iterators. The reason for using this table is to avoid storing pointers
 // in the YARA's VM stack. Instead of the pointers we store an index within
@@ -335,12 +404,16 @@ static YR_ITERATOR_NEXT_FUNC iter_next_func_table[] = {
     iter_dict_next,
     iter_int_range_next,
     iter_int_enum_next,
+    iter_string_set_next,
+    iter_text_string_set_next,
 };
 
-#define ITER_NEXT_ARRAY     0
-#define ITER_NEXT_DICT      1
-#define ITER_NEXT_INT_RANGE 2
-#define ITER_NEXT_INT_ENUM  3
+#define ITER_NEXT_ARRAY           0
+#define ITER_NEXT_DICT            1
+#define ITER_NEXT_INT_RANGE       2
+#define ITER_NEXT_INT_ENUM        3
+#define ITER_NEXT_STRING_SET      4
+#define ITER_NEXT_TEXT_STRING_SET 5
 
 int yr_execute_code(YR_SCAN_CONTEXT* context)
 {
@@ -529,6 +602,78 @@ int yr_execute_code(YR_SCAN_CONTEXT* context)
       stop = (result != ERROR_SUCCESS);
       break;
 
+    case OP_ITER_START_STRING_SET:
+      YR_DEBUG_FPRINTF(
+          2,
+          stderr,
+          "- case OP_ITER_START_STRING_SET: // %s()\n",
+          __FUNCTION__);
+
+      pop(r1);
+
+      r3.p = yr_notebook_alloc(
+          it_notebook,
+          sizeof(YR_ITERATOR) + sizeof(YR_STRING*) * (size_t) r1.i);
+
+      if (r3.p == NULL)
+      {
+        result = ERROR_INSUFFICIENT_MEMORY;
+      }
+      else
+      {
+        r3.it->string_set_it.count = r1.i;
+        r3.it->string_set_it.index = 0;
+        r3.it->next_func_idx = ITER_NEXT_STRING_SET;
+
+        for (int64_t i = r1.i; i > 0; i--)
+        {
+          pop(r2);
+          r3.it->string_set_it.strings[i - 1] = r2.s;
+        }
+
+        // One last pop of the UNDEFINED string
+        pop(r2);
+        push(r3);
+      }
+
+      stop = (result != ERROR_SUCCESS);
+      break;
+
+    case OP_ITER_START_TEXT_STRING_SET:
+      YR_DEBUG_FPRINTF(
+          2,
+          stderr,
+          "- case OP_ITER_START_TEXT_STRING_SET: // %s()\n",
+          __FUNCTION__);
+
+      pop(r1);
+
+      r3.p = yr_notebook_alloc(
+          it_notebook,
+          sizeof(YR_ITERATOR) + sizeof(SIZED_STRING*) * (size_t) r1.i);
+
+      if (r3.p == NULL)
+      {
+        result = ERROR_INSUFFICIENT_MEMORY;
+      }
+      else
+      {
+        r3.it->text_string_set_it.count = r1.i;
+        r3.it->text_string_set_it.index = 0;
+        r3.it->next_func_idx = ITER_NEXT_TEXT_STRING_SET;
+
+        for (int64_t i = r1.i; i > 0; i--)
+        {
+          pop(r2);
+          r3.it->text_string_set_it.strings[i - 1] = r2.ss;
+        }
+
+        push(r3);
+      }
+
+      stop = (result != ERROR_SUCCESS);
+      break;
+
     case OP_ITER_NEXT:
       YR_DEBUG_FPRINTF(
           2, stderr, "- case OP_ITER_NEXT: // %s()\n", __FUNCTION__);
@@ -554,9 +699,77 @@ int yr_execute_code(YR_SCAN_CONTEXT* context)
       stop = (result != ERROR_SUCCESS);
       break;
 
+    case OP_ITER_CONDITION:
+      YR_DEBUG_FPRINTF(
+          2, stderr, "- case OP_ITER_CONDITION: // %s()\n", __FUNCTION__);
+
+      // Evaluate the iteration condition of the loop. This instruction
+      // evaluates to 1 if the loop should continue and 0 if it shouldn't
+      // (due to short-circuit evaluation).
+
+      pop(r2);  // min. expression - all, any, none, integer
+      pop(r3);  // number of true expressions
+      pop(r4);  // last expression result
+
+      // In case of 'all' loop, end once we the body failed
+      if (is_undef(r2))
+      {
+        r1.i = r4.i != 0 ? 1 : 0;
+      }
+      // In case of 'none' loop, end once the body succeed
+      else if (r2.i == 0)
+      {
+        r1.i = r4.i != 1 ? 1 : 0;
+      }
+      // In case of other loops, end once we satified min. expr.
+      else
+      {
+        r1.i = r3.i + r4.i < r2.i ? 1 : 0;
+      }
+
+      // Push whether loop should continue and repush
+      // the last expression result
+      push(r1);
+      push(r4);
+      break;
+
+    case OP_ITER_END:
+      YR_DEBUG_FPRINTF(
+          2, stderr, "- case OP_ITER_END: // %s()\n", __FUNCTION__);
+
+      // Evaluate the whole loop. Whether it was successful or not
+      // and whether it satisfied it's quantifier.
+
+      pop(r2);  // min. expression - all, any, none, integer
+      pop(r3);  // number of true expressions
+      pop(r4);  // number of total iterations
+
+      // If there was 0 iterations in total, it doesn't
+      // matter what other numbers show. We can't evaluate
+      // the loop as true.
+      if (r4.i == 0)
+      {
+        r1.i = 0;
+      }
+      else if (is_undef(r2))
+      {
+        r1.i = r3.i == r4.i ? 1 : 0;
+      }
+      else if (r2.i == 0)
+      {
+        r1.i = r3.i == 0 ? 1 : 0;
+      }
+      else
+      {
+        r1.i = r3.i >= r2.i ? 1 : 0;
+      }
+
+      push(r1);
+      break;
+
     case OP_PUSH:
       YR_DEBUG_FPRINTF(2, stderr, "- case OP_PUSH: // %s()\n", __FUNCTION__);
-      memcpy(&r1.i, ip, sizeof(uint64_t));
+      r1.i = yr_unaligned_u64(ip);
       ip += sizeof(uint64_t);
       push(r1);
       break;
@@ -574,7 +787,7 @@ int yr_execute_code(YR_SCAN_CONTEXT* context)
       break;
 
     case OP_PUSH_16:
-      r1.i = *(uint16_t*) (ip);
+      r1.i = yr_unaligned_u16(ip);
       YR_DEBUG_FPRINTF(
           2,
           stderr,
@@ -586,7 +799,7 @@ int yr_execute_code(YR_SCAN_CONTEXT* context)
       break;
 
     case OP_PUSH_32:
-      r1.i = *(uint32_t*) (ip);
+      r1.i = yr_unaligned_u32(ip);
       YR_DEBUG_FPRINTF(
           2,
           stderr,
@@ -610,7 +823,7 @@ int yr_execute_code(YR_SCAN_CONTEXT* context)
 
     case OP_CLEAR_M:
       YR_DEBUG_FPRINTF(2, stderr, "- case OP_CLEAR_M: // %s()\n", __FUNCTION__);
-      memcpy(&r1.i, ip, sizeof(uint64_t));
+      r1.i = yr_unaligned_u64(ip);
       ip += sizeof(uint64_t);
 #if YR_PARANOID_EXEC
       ensure_within_mem(r1.i);
@@ -620,7 +833,7 @@ int yr_execute_code(YR_SCAN_CONTEXT* context)
 
     case OP_ADD_M:
       YR_DEBUG_FPRINTF(2, stderr, "- case OP_ADD_M: // %s()\n", __FUNCTION__);
-      memcpy(&r1.i, ip, sizeof(uint64_t));
+      r1.i = yr_unaligned_u64(ip);
       ip += sizeof(uint64_t);
 #if YR_PARANOID_EXEC
       ensure_within_mem(r1.i);
@@ -632,7 +845,7 @@ int yr_execute_code(YR_SCAN_CONTEXT* context)
 
     case OP_INCR_M:
       YR_DEBUG_FPRINTF(2, stderr, "- case OP_INCR_M: // %s()\n", __FUNCTION__);
-      memcpy(&r1.i, ip, sizeof(uint64_t));
+      r1.i = yr_unaligned_u64(ip);
       ip += sizeof(uint64_t);
 #if YR_PARANOID_EXEC
       ensure_within_mem(r1.i);
@@ -642,7 +855,7 @@ int yr_execute_code(YR_SCAN_CONTEXT* context)
 
     case OP_PUSH_M:
       YR_DEBUG_FPRINTF(2, stderr, "- case OP_PUSH_M: // %s()\n", __FUNCTION__);
-      memcpy(&r1.i, ip, sizeof(uint64_t));
+      r1.i = yr_unaligned_u64(ip);
       ip += sizeof(uint64_t);
 #if YR_PARANOID_EXEC
       ensure_within_mem(r1.i);
@@ -653,7 +866,7 @@ int yr_execute_code(YR_SCAN_CONTEXT* context)
 
     case OP_POP_M:
       YR_DEBUG_FPRINTF(2, stderr, "- case OP_POP_M: // %s()\n", __FUNCTION__);
-      memcpy(&r1.i, ip, sizeof(uint64_t));
+      r1.i = yr_unaligned_u64(ip);
       ip += sizeof(uint64_t);
 #if YR_PARANOID_EXEC
       ensure_within_mem(r1.i);
@@ -664,7 +877,7 @@ int yr_execute_code(YR_SCAN_CONTEXT* context)
 
     case OP_SET_M:
       YR_DEBUG_FPRINTF(2, stderr, "- case OP_SET_M: // %s()\n", __FUNCTION__);
-      memcpy(&r1.i, ip, sizeof(uint64_t));
+      r1.i = yr_unaligned_u64(ip);
       ip += sizeof(uint64_t);
 #if YR_PARANOID_EXEC
       ensure_within_mem(r1.i);
@@ -678,7 +891,7 @@ int yr_execute_code(YR_SCAN_CONTEXT* context)
     case OP_SWAPUNDEF:
       YR_DEBUG_FPRINTF(
           2, stderr, "- case OP_SWAPUNDEF: // %s()\n", __FUNCTION__);
-      memcpy(&r1.i, ip, sizeof(uint64_t));
+      r1.i = yr_unaligned_u64(ip);
       ip += sizeof(uint64_t);
 #if YR_PARANOID_EXEC
       ensure_within_mem(r1.i);
@@ -836,10 +1049,12 @@ int yr_execute_code(YR_SCAN_CONTEXT* context)
       pop(r1);
       ensure_defined(r2);
       ensure_defined(r1);
-      if (r2.i != 0)
-        r1.i = r1.i % r2.i;
-      else
+      // If divisor is zero the result is undefined. It's also undefined
+      // when dividing INT64_MIN by -1.
+      if (r2.i == 0 || (r1.i == INT64_MIN && r2.i == -1))
         r1.i = YR_UNDEFINED;
+      else
+        r1.i = r1.i % r2.i;
       push(r1);
       break;
 
@@ -918,7 +1133,7 @@ int yr_execute_code(YR_SCAN_CONTEXT* context)
     case OP_PUSH_RULE:
       YR_DEBUG_FPRINTF(
           2, stderr, "- case OP_PUSH_RULE: // %s()\n", __FUNCTION__);
-      memcpy(&r1.i, ip, sizeof(uint64_t));
+      r1.i = yr_unaligned_u64(ip);
       ip += sizeof(uint64_t);
 
       rule = &context->rules->rules_table[r1.i];
@@ -941,21 +1156,38 @@ int yr_execute_code(YR_SCAN_CONTEXT* context)
     case OP_INIT_RULE:
       YR_DEBUG_FPRINTF(
           2, stderr, "- case OP_INIT_RULE: // %s()\n", __FUNCTION__);
+
       // After the opcode there's an int32_t corresponding to the jump's
       // offset and an uint32_t corresponding to the rule's index.
-      current_rule_idx = *(uint32_t*) (ip + sizeof(int32_t));
+      current_rule_idx = yr_unaligned_u32(ip + sizeof(int32_t));
 
+      // The curent rule index can't be larger than the number of rules.
       assert(current_rule_idx < context->rules->num_rules);
 
       current_rule = &context->rules->rules_table[current_rule_idx];
 
-      // If the rule is disabled let's skip its code.
-      ip = jmp_if(RULE_IS_DISABLED(current_rule), ip);
+      // If the rule is disabled, let's skip its code.
+      bool skip_rule = RULE_IS_DISABLED(current_rule);
 
-      // Skip the bytes corresponding to the rule's index, but only if not
-      // taking the jump.
-      if (!RULE_IS_DISABLED(current_rule))
+      // The rule is also skipped if it is not required to be evaluated.
+      skip_rule |= yr_bitmask_is_not_set(
+          context->required_eval, current_rule_idx);
+
+      ip = jmp_if(skip_rule, ip);
+
+      if (skip_rule)
+      {
+        // If the rule is skipped it is false, and if a global rule is false
+        // we must mark its namespace as unsatisfied.
+        if (RULE_IS_GLOBAL(current_rule))
+          yr_bitmask_set(context->ns_unsatisfied_flags, current_rule->ns->idx);
+      }
+      else
+      {
+        // If not taking the jump, skip the bytes corresponding to the
+        // rule's index.
         ip += sizeof(uint32_t);
+      }
 
       break;
 
@@ -964,7 +1196,7 @@ int yr_execute_code(YR_SCAN_CONTEXT* context)
           2, stderr, "- case OP_MATCH_RULE: // %s()\n", __FUNCTION__);
       pop(r1);
 
-      memcpy(&r2.i, ip, sizeof(uint64_t));
+      r2.i = yr_unaligned_u64(ip);
       ip += sizeof(uint64_t);
 
       rule = &context->rules->rules_table[r2.i];
@@ -990,7 +1222,8 @@ int yr_execute_code(YR_SCAN_CONTEXT* context)
     case OP_OBJ_LOAD:
       YR_DEBUG_FPRINTF(
           2, stderr, "- case OP_OBJ_LOAD: // %s()\n", __FUNCTION__);
-      identifier = *(char**) (ip);
+
+      identifier = yr_unaligned_char_ptr(ip);
       ip += sizeof(uint64_t);
 
 #if YR_PARANOID_EXEC
@@ -1007,7 +1240,8 @@ int yr_execute_code(YR_SCAN_CONTEXT* context)
     case OP_OBJ_FIELD:
       YR_DEBUG_FPRINTF(
           2, stderr, "- case OP_OBJ_FIELD: // %s()\n", __FUNCTION__);
-      identifier = *(char**) (ip);
+
+      identifier = yr_unaligned_char_ptr(ip);
       ip += sizeof(uint64_t);
 
 #if YR_PARANOID_EXEC
@@ -1114,7 +1348,8 @@ int yr_execute_code(YR_SCAN_CONTEXT* context)
 
     case OP_CALL:
       YR_DEBUG_FPRINTF(2, stderr, "- case OP_CALL: // %s()\n", __FUNCTION__);
-      args_fmt = *(char**) (ip);
+
+      args_fmt = yr_unaligned_char_ptr(ip);
       ip += sizeof(uint64_t);
 
       int i = (int) strlen(args_fmt);
@@ -1192,6 +1427,10 @@ int yr_execute_code(YR_SCAN_CONTEXT* context)
         result = yr_arena_write_data(obj_arena, 0, &r1.o, sizeof(r1.o), NULL);
         obj_count++;
       }
+      else
+      {
+        r1.i = YR_UNDEFINED;
+      }
 
       stop = (result != ERROR_SUCCESS);
       push(r1);
@@ -1215,12 +1454,7 @@ int yr_execute_code(YR_SCAN_CONTEXT* context)
       pop(r2);
       pop(r1);
 
-      if (is_undef(r1))
-      {
-        r1.i = 0;
-        push(r1);
-        break;
-      }
+      ensure_defined(r1);
 
 #if YR_PARANOID_EXEC
       ensure_within_rules_arena(r2.p);
@@ -1303,7 +1537,7 @@ int yr_execute_code(YR_SCAN_CONTEXT* context)
       ensure_defined(r2);
 
 #if YR_PARANOID_EXEC
-      ensure_within_rules_arena(r1.p);
+      ensure_within_rules_arena(r3.p);
 #endif
 
       match = context->matches[r3.s->idx].head;
@@ -1384,7 +1618,7 @@ int yr_execute_code(YR_SCAN_CONTEXT* context)
 
     case OP_OF:
     case OP_OF_PERCENT:
-      memcpy(&r2.i, ip, sizeof(uint64_t));
+      r2.i = yr_unaligned_u64(ip);
       ip += sizeof(uint64_t);
       assert(r2.i == OF_STRING_SET || r2.i == OF_RULE_SET);
       found = 0;
@@ -1493,7 +1727,7 @@ int yr_execute_code(YR_SCAN_CONTEXT* context)
           }
 
           // If current match is past range end, we can stop as matches
-          // are sortred by offset in increasing order, so all remaining
+          // are sorted by offset in increasing order, so all remaining
           // matches are part the range end too.
           if (match->base + match->offset > r1.i)
             break;
@@ -1529,6 +1763,81 @@ int yr_execute_code(YR_SCAN_CONTEXT* context)
       push(r1);
       break;
 
+    case OP_OF_FOUND_AT:
+      YR_DEBUG_FPRINTF(
+          2, stderr, "- case OP_OF_FOUND_AT: // %s()\n", __FUNCTION__);
+
+      found = 0;
+      count = 0;
+
+      pop(r2);  // Match location
+      pop(r1);  // First string
+
+      // Match location must be defined.
+      if (is_undef(r2))
+      {
+        // Remove all the strings.
+        while (!is_undef(r1)) pop(r1);
+        // Remove the quantifier at the bottom of the stack.
+        pop(r1);
+        r1.i = YR_UNDEFINED;
+        push(r1);
+        break;
+      }
+
+      while (!is_undef(r1))
+      {
+#if YR_PARANOID_EXEC
+        ensure_within_rules_arena(r1.p);
+#endif
+        match = context->matches[r1.s->idx].head;
+
+        while (match != NULL)
+        {
+          // String match at the desired location?
+          if (match->base + match->offset == r2.i)
+          {
+            found++;
+            break;
+          }
+
+          // If current match is past desired location, we can stop as matches
+          // are sorted by offset in increasing order, so all remaining
+          // matches are past it.
+          if (match->base + match->offset > r2.i)
+            break;
+
+          match = match->next;
+        }
+
+        count++;
+        pop(r1);
+      }
+
+      pop(r2);  // Quantifier X in expressions like "X of string_set in range"
+
+      // Quantifier is "all".
+      if (is_undef(r2))
+      {
+        r1.i = found >= count ? 1 : 0;
+      }
+      // Quantifier is 0 or none. This is a special case in which we want
+      // exactly 0 strings matching. More information at:
+      // https://github.com/VirusTotal/yara/issues/1695
+      else if (r2.i == 0)
+      {
+        r1.i = found == 0 ? 1 : 0;
+      }
+      // In all other cases the number of strings matching should be at least
+      // the amount specified by the quantifier.
+      else
+      {
+        r1.i = found >= r2.i ? 1 : 0;
+      }
+
+      push(r1);
+      break;
+
     case OP_FILESIZE:
       r1.i = context->file_size;
       YR_DEBUG_FPRINTF(
@@ -1636,7 +1945,7 @@ int yr_execute_code(YR_SCAN_CONTEXT* context)
 
     case OP_IMPORT:
       YR_DEBUG_FPRINTF(2, stderr, "- case OP_IMPORT: // %s()\n", __FUNCTION__);
-      memcpy(&r1.i, ip, sizeof(uint64_t));
+      r1.i = yr_unaligned_u64(ip);
       ip += sizeof(uint64_t);
 
 #if YR_PARANOID_EXEC
@@ -1686,7 +1995,7 @@ int yr_execute_code(YR_SCAN_CONTEXT* context)
     case OP_INT_TO_DBL:
       YR_DEBUG_FPRINTF(
           2, stderr, "- case OP_INT_TO_DBL: // %s()\n", __FUNCTION__);
-      memcpy(&r1.i, ip, sizeof(uint64_t));
+      r1.i = yr_unaligned_u64(ip);
       ip += sizeof(uint64_t);
 
 #if YR_PARANOID_EXEC
@@ -1811,10 +2120,12 @@ int yr_execute_code(YR_SCAN_CONTEXT* context)
       pop(r1);
       ensure_defined(r2);
       ensure_defined(r1);
-      if (r2.i != 0)
-        r1.i = r1.i / r2.i;
-      else
+      // If divisor is zero the result is undefined. It's also undefined
+      // when dividing INT64_MIN by -1.
+      if (r2.i == 0 || (r1.i == INT64_MIN && r2.i == -1))
         r1.i = YR_UNDEFINED;
+      else
+        r1.i = r1.i / r2.i;
       push(r1);
       break;
 
