@@ -33,8 +33,6 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #include <yara/endian.h>
 #include <yara/types.h>
 
-#pragma pack(push, 1)
-
 #if defined(_WIN32) || defined(__CYGWIN__)
 #include <windows.h>
 
@@ -52,6 +50,18 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #define IMAGE_FILE_MACHINE_ARM64 0xaa64
 #endif
 
+#ifndef IMAGE_SUBSYSTEM_EFI_ROM_IMAGE
+#define IMAGE_SUBSYSTEM_EFI_ROM_IMAGE 13
+#endif
+
+#ifndef IMAGE_DIRECTORY_ENTRY_COPYRIGHT
+#define IMAGE_DIRECTORY_ENTRY_COPYRIGHT 7  // (X86 usage)
+#endif
+
+#ifndef IMAGE_FILE_MACHINE_TARGET_HOST
+#define IMAGE_FILE_MACHINE_TARGET_HOST 0x0001
+#endif
+
 #else
 
 #include <stdlib.h>
@@ -59,6 +69,8 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
 typedef uint8_t BYTE;
 typedef uint16_t WORD;
+typedef uint16_t WCHAR;
+typedef int16_t SHORT;
 typedef uint32_t DWORD;
 typedef int32_t LONG;
 typedef uint32_t ULONG;
@@ -81,7 +93,7 @@ typedef uint64_t ULONGLONG;
 
 #endif
 
-#pragma pack(push, 2)
+#pragma pack(push, 1)
 
 typedef struct _IMAGE_DOS_HEADER
 {                   // DOS .EXE header
@@ -106,14 +118,6 @@ typedef struct _IMAGE_DOS_HEADER
   LONG e_lfanew;    // File address of new exe header
 } IMAGE_DOS_HEADER, *PIMAGE_DOS_HEADER;
 
-#pragma pack(pop)
-
-//
-// File header format.
-//
-
-#pragma pack(push, 4)
-
 typedef struct _IMAGE_FILE_HEADER
 {
   WORD Machine;
@@ -180,13 +184,50 @@ typedef struct _IMAGE_FILE_HEADER
 #define IMAGE_FILE_MACHINE_SH5           0x01a8
 #define IMAGE_FILE_MACHINE_THUMB         0x01c2
 #define IMAGE_FILE_MACHINE_WCEMIPSV2     0x0169
+// Useful for indicating we want to interact with the host and not a WoW guest.
+#define IMAGE_FILE_MACHINE_TARGET_HOST   0x0001
+// MIPS little-endian, 0x160 big-endian
+#define IMAGE_FILE_MACHINE_R3000         0x0162  // MIPS little-endian
+#define IMAGE_FILE_MACHINE_R10000        0x0168  // Alpha_AXP
+#define IMAGE_FILE_MACHINE_ALPHA         0x0184  // SH3E little-endian
+#define IMAGE_FILE_MACHINE_SH3E          0x01a4  // ALPHA64
+#define IMAGE_FILE_MACHINE_ALPHA64       0x0284
+#define IMAGE_FILE_MACHINE_AXP64         IMAGE_FILE_MACHINE_ALPHA64
+#define IMAGE_FILE_MACHINE_TRICORE       0x0520  // Infineon
+#define IMAGE_FILE_MACHINE_CEF           0x0CEF
+#define IMAGE_FILE_MACHINE_CEE           0xC0EE
 
 // Section characteristics
+#define IMAGE_SCN_TYPE_NO_PAD            0x00000008
 #define IMAGE_SCN_CNT_CODE               0x00000020
 #define IMAGE_SCN_CNT_INITIALIZED_DATA   0x00000040
 #define IMAGE_SCN_CNT_UNINITIALIZED_DATA 0x00000080
+#define IMAGE_SCN_LNK_OTHER              0x00000100
+#define IMAGE_SCN_LNK_INFO               0x00000200
+#define IMAGE_SCN_LNK_REMOVE             0x00000800
+#define IMAGE_SCN_LNK_COMDAT             0x00001000
+#define IMAGE_SCN_NO_DEFER_SPEC_EXC      0x00004000
 #define IMAGE_SCN_GPREL                  0x00008000
+#define IMAGE_SCN_MEM_FARDATA            0x00008000
+#define IMAGE_SCN_MEM_PURGEABLE          0x00020000
 #define IMAGE_SCN_MEM_16BIT              0x00020000
+#define IMAGE_SCN_MEM_LOCKED             0x00040000
+#define IMAGE_SCN_MEM_PRELOAD            0x00080000
+#define IMAGE_SCN_ALIGN_1BYTES           0x00100000
+#define IMAGE_SCN_ALIGN_2BYTES           0x00200000
+#define IMAGE_SCN_ALIGN_4BYTES           0x00300000
+#define IMAGE_SCN_ALIGN_8BYTES           0x00400000
+#define IMAGE_SCN_ALIGN_16BYTES          0x00500000
+#define IMAGE_SCN_ALIGN_32BYTES          0x00600000
+#define IMAGE_SCN_ALIGN_64BYTES          0x00700000
+#define IMAGE_SCN_ALIGN_128BYTES         0x00800000
+#define IMAGE_SCN_ALIGN_256BYTES         0x00900000
+#define IMAGE_SCN_ALIGN_512BYTES         0x00A00000
+#define IMAGE_SCN_ALIGN_1024BYTES        0x00B00000
+#define IMAGE_SCN_ALIGN_2048BYTES        0x00C00000
+#define IMAGE_SCN_ALIGN_4096BYTES        0x00D00000
+#define IMAGE_SCN_ALIGN_8192BYTES        0x00E00000
+#define IMAGE_SCN_ALIGN_MASK             0x00F00000
 #define IMAGE_SCN_LNK_NRELOC_OVFL        0x01000000
 #define IMAGE_SCN_MEM_DISCARDABLE        0x02000000
 #define IMAGE_SCN_MEM_NOT_CACHED         0x04000000
@@ -195,6 +236,7 @@ typedef struct _IMAGE_FILE_HEADER
 #define IMAGE_SCN_MEM_EXECUTE            0x20000000
 #define IMAGE_SCN_MEM_READ               0x40000000
 #define IMAGE_SCN_MEM_WRITE              0x80000000
+#define IMAGE_SCN_SCALE_INDEX            0x00000001
 
 //
 // Directory format.
@@ -302,6 +344,7 @@ typedef struct _IMAGE_OPTIONAL_HEADER64
 
 #define IMAGE_NT_OPTIONAL_HDR32_MAGIC 0x10b
 #define IMAGE_NT_OPTIONAL_HDR64_MAGIC 0x20b
+#define IMAGE_ROM_OPTIONAL_HDR_MAGIC  0x107
 
 typedef struct _IMAGE_NT_HEADERS32
 {
@@ -322,7 +365,7 @@ typedef struct _IMAGE_NT_HEADERS64
 // IMAGE_FIRST_SECTION doesn't need 32/64 versions since the file header is
 // the same either way.
 
-#define IMAGE_FIRST_SECTION(ntheader)                                   \
+#define IMAGE_FIRST_SECTION(ntheader) \
   ((PIMAGE_SECTION_HEADER)(                                             \
       (BYTE*) ntheader + offsetof(IMAGE_NT_HEADERS32, OptionalHeader) + \
       yr_le16toh(((PIMAGE_NT_HEADERS32)(ntheader))                      \
@@ -347,13 +390,16 @@ typedef struct _IMAGE_NT_HEADERS64
 
 // DllCharacteristics values
 
+#define IMAGE_DLLCHARACTERISTICS_HIGH_ENTROPY_VA       0x0020
 #define IMAGE_DLLCHARACTERISTICS_DYNAMIC_BASE          0x0040
 #define IMAGE_DLLCHARACTERISTICS_FORCE_INTEGRITY       0x0080
 #define IMAGE_DLLCHARACTERISTICS_NX_COMPAT             0x0100
 #define IMAGE_DLLCHARACTERISTICS_NO_ISOLATION          0x0200
 #define IMAGE_DLLCHARACTERISTICS_NO_SEH                0x0400
 #define IMAGE_DLLCHARACTERISTICS_NO_BIND               0x0800
+#define IMAGE_DLLCHARACTERISTICS_APPCONTAINER          0x1000
 #define IMAGE_DLLCHARACTERISTICS_WDM_DRIVER            0x2000
+#define IMAGE_DLLCHARACTERISTICS_GUARD_CF              0x4000
 #define IMAGE_DLLCHARACTERISTICS_TERMINAL_SERVER_AWARE 0x8000
 
 //
@@ -434,6 +480,50 @@ typedef struct _IMAGE_THUNK_DATA32
 #define IMAGE_ORDINAL_FLAG32 0x80000000
 #define IMAGE_ORDINAL_FLAG64 0x8000000000000000L
 
+typedef struct _IMAGE_BOUND_IMPORT_DESCRIPTOR
+{
+  DWORD TimeDateStamp;
+  WORD OffsetModuleName;
+  WORD NumberOfModuleForwarderRefs;
+  // Array of zero or more IMAGE_BOUND_FORWARDER_REF follows
+} IMAGE_BOUND_IMPORT_DESCRIPTOR, *PIMAGE_BOUND_IMPORT_DESCRIPTOR;
+
+typedef struct _IMAGE_BOUND_FORWARDER_REF
+{
+  DWORD TimeDateStamp;
+  WORD OffsetModuleName;
+  WORD Reserved;
+} IMAGE_BOUND_FORWARDER_REF, *PIMAGE_BOUND_FORWARDER_REF;
+
+typedef struct _IMAGE_DELAYLOAD_DESCRIPTOR
+{
+  union
+  {
+    DWORD AllAttributes;
+    struct
+    {
+      DWORD RvaBased : 1;  // Delay load version 2
+      DWORD ReservedAttributes : 31;
+    } DUMMYSTRUCTNAME;
+  } Attributes;
+
+  // RVA to the name of the target library (NULL-terminate ASCII string)
+  DWORD DllNameRVA;
+  // RVA to the HMODULE caching location (PHMODULE)
+  DWORD ModuleHandleRVA;
+  // RVA to the start of the IAT (PIMAGE_THUNK_DATA)
+  DWORD ImportAddressTableRVA;
+  // RVA to the start of the name table (PIMAGE_THUNK_DATA::AddressOfData)
+  DWORD ImportNameTableRVA;
+  // RVA to an optional bound IAT
+  DWORD BoundImportAddressTableRVA;
+  // RVA to an optional unload info table
+  DWORD UnloadInformationTableRVA;
+  // 0 if not bound, otherwise, date/time of the target DLL
+  DWORD TimeDateStamp;
+
+} IMAGE_DELAYLOAD_DESCRIPTOR, *PIMAGE_DELAYLOAD_DESCRIPTOR;
+
 typedef struct _IMAGE_THUNK_DATA64
 {
   union
@@ -446,6 +536,12 @@ typedef struct _IMAGE_THUNK_DATA64
 
 } IMAGE_THUNK_DATA64, *PIMAGE_THUNK_DATA64;
 
+typedef struct _IMAGE_RESOURCE_DIR_STRING_U
+{
+  WORD Length;
+  WCHAR NameString[1];
+} IMAGE_RESOURCE_DIR_STRING_U, *PIMAGE_RESOURCE_DIR_STRING_U;
+
 typedef struct _IMAGE_RESOURCE_DIRECTORY_ENTRY
 {
   DWORD Name;
@@ -470,14 +566,20 @@ typedef struct _IMAGE_RESOURCE_DIRECTORY
   WORD NumberOfIdEntries;
 } IMAGE_RESOURCE_DIRECTORY, *PIMAGE_RESOURCE_DIRECTORY;
 
-#define IMAGE_DEBUG_TYPE_UNKNOWN   0
-#define IMAGE_DEBUG_TYPE_COFF      1
-#define IMAGE_DEBUG_TYPE_CODEVIEW  2
-#define IMAGE_DEBUG_TYPE_FPO       3
-#define IMAGE_DEBUG_TYPE_MISC      4
-#define IMAGE_DEBUG_TYPE_EXCEPTION 5
-#define IMAGE_DEBUG_TYPE_FIXUP     6
-#define IMAGE_DEBUG_TYPE_BORLAND   9
+#define IMAGE_DEBUG_TYPE_FPO           3
+#define IMAGE_DEBUG_TYPE_MISC          4
+#define IMAGE_DEBUG_TYPE_EXCEPTION     5
+#define IMAGE_DEBUG_TYPE_FIXUP         6
+#define IMAGE_DEBUG_TYPE_OMAP_TO_SRC   7
+#define IMAGE_DEBUG_TYPE_OMAP_FROM_SRC 8
+#define IMAGE_DEBUG_TYPE_BORLAND       9
+#define IMAGE_DEBUG_TYPE_RESERVED10    10
+#define IMAGE_DEBUG_TYPE_CLSID         11
+#define IMAGE_DEBUG_TYPE_VC_FEATURE    12
+#define IMAGE_DEBUG_TYPE_POGO          13
+#define IMAGE_DEBUG_TYPE_ILTCG         14
+#define IMAGE_DEBUG_TYPE_MPX           15
+#define IMAGE_DEBUG_TYPE_REPRO         16
 
 typedef struct _IMAGE_DEBUG_DIRECTORY
 {
@@ -491,12 +593,183 @@ typedef struct _IMAGE_DEBUG_DIRECTORY
   DWORD PointerToRawData;
 } IMAGE_DEBUG_DIRECTORY, *PIMAGE_DEBUG_DIRECTORY;
 
+//
+// Symbol format.
+//
+
+typedef struct _IMAGE_SYMBOL
+{
+  union
+  {
+    BYTE ShortName[8];
+    struct
+    {
+      DWORD Short;  // if 0, use LongName
+      DWORD Long;   // offset into string table
+    } Name;
+    DWORD LongName[2];  // PBYTE [2]
+  } N;
+  DWORD Value;
+  SHORT SectionNumber;
+  WORD Type;
+  BYTE StorageClass;
+  BYTE NumberOfAuxSymbols;
+} IMAGE_SYMBOL, *PIMAGE_SYMBOL;
+
+#define IMAGE_SIZEOF_SYMBOL 18
+
+typedef struct _IMAGE_SYMBOL_EX
+{
+  union
+  {
+    BYTE ShortName[8];
+    struct
+    {
+      DWORD Short;  // if 0, use LongName
+      DWORD Long;   // offset into string table
+    } Name;
+    DWORD LongName[2];  // PBYTE  [2]
+  } N;
+  DWORD Value;
+  LONG SectionNumber;
+  WORD Type;
+  BYTE StorageClass;
+  BYTE NumberOfAuxSymbols;
+} IMAGE_SYMBOL_EX, *PIMAGE_SYMBOL_EX;
+
+//
+// Section values.
+//
+// Symbols have a section number of the section in which they are
+// defined. Otherwise, section numbers have the following meanings:
+//
+
+#define IMAGE_SYM_UNDEFINED      (SHORT) 0  // Symbol is undefined or is common.
+#define IMAGE_SYM_ABSOLUTE       (SHORT) - 1  // Symbol is an absolute value.
+#define IMAGE_SYM_DEBUG          (SHORT) - 2  // Symbol is a special debug item.
+#define IMAGE_SYM_SECTION_MAX    0xFEFF  // Values 0xFF00-0xFFFF are special
+#define IMAGE_SYM_SECTION_MAX_EX MAXLONG
+
+//
+// Type (fundamental) values.
+//
+
+#define IMAGE_SYM_TYPE_NULL   0x0000  // no type.
+#define IMAGE_SYM_TYPE_VOID   0x0001  //
+#define IMAGE_SYM_TYPE_CHAR   0x0002  // type character.
+#define IMAGE_SYM_TYPE_SHORT  0x0003  // type short integer.
+#define IMAGE_SYM_TYPE_INT    0x0004  //
+#define IMAGE_SYM_TYPE_LONG   0x0005  //
+#define IMAGE_SYM_TYPE_FLOAT  0x0006  //
+#define IMAGE_SYM_TYPE_DOUBLE 0x0007  //
+#define IMAGE_SYM_TYPE_STRUCT 0x0008  //
+#define IMAGE_SYM_TYPE_UNION  0x0009  //
+#define IMAGE_SYM_TYPE_ENUM   0x000A  // enumeration.
+#define IMAGE_SYM_TYPE_MOE    0x000B  // member of enumeration.
+#define IMAGE_SYM_TYPE_BYTE   0x000C  //
+#define IMAGE_SYM_TYPE_WORD   0x000D  //
+#define IMAGE_SYM_TYPE_UINT   0x000E  //
+#define IMAGE_SYM_TYPE_DWORD  0x000F  //
+#define IMAGE_SYM_TYPE_PCODE  0x8000  //
+//
+// Type (derived) values.
+//
+
+#define IMAGE_SYM_DTYPE_NULL             0  // no derived type.
+#define IMAGE_SYM_DTYPE_POINTER          1  // pointer.
+#define IMAGE_SYM_DTYPE_FUNCTION         2  // function.
+#define IMAGE_SYM_DTYPE_ARRAY            3  // array.
+
+//
+// Storage classes.
+//
+#define IMAGE_SYM_CLASS_END_OF_FUNCTION  (BYTE) - 1
+#define IMAGE_SYM_CLASS_NULL             0x0000
+#define IMAGE_SYM_CLASS_AUTOMATIC        0x0001
+#define IMAGE_SYM_CLASS_EXTERNAL         0x0002
+#define IMAGE_SYM_CLASS_STATIC           0x0003
+#define IMAGE_SYM_CLASS_REGISTER         0x0004
+#define IMAGE_SYM_CLASS_EXTERNAL_DEF     0x0005
+#define IMAGE_SYM_CLASS_LABEL            0x0006
+#define IMAGE_SYM_CLASS_UNDEFINED_LABEL  0x0007
+#define IMAGE_SYM_CLASS_MEMBER_OF_STRUCT 0x0008
+#define IMAGE_SYM_CLASS_ARGUMENT         0x0009
+#define IMAGE_SYM_CLASS_STRUCT_TAG       0x000A
+#define IMAGE_SYM_CLASS_MEMBER_OF_UNION  0x000B
+#define IMAGE_SYM_CLASS_UNION_TAG        0x000C
+#define IMAGE_SYM_CLASS_TYPE_DEFINITION  0x000D
+#define IMAGE_SYM_CLASS_UNDEFINED_STATIC 0x000E
+#define IMAGE_SYM_CLASS_ENUM_TAG         0x000F
+#define IMAGE_SYM_CLASS_MEMBER_OF_ENUM   0x0010
+#define IMAGE_SYM_CLASS_REGISTER_PARAM   0x0011
+#define IMAGE_SYM_CLASS_BIT_FIELD        0x0012
+
+#define IMAGE_SYM_CLASS_FAR_EXTERNAL 0x0044  //
+
+#define IMAGE_SYM_CLASS_BLOCK         0x0064
+#define IMAGE_SYM_CLASS_FUNCTION      0x0065
+#define IMAGE_SYM_CLASS_END_OF_STRUCT 0x0066
+#define IMAGE_SYM_CLASS_FILE          0x0067
+// new
+#define IMAGE_SYM_CLASS_SECTION       0x0068
+#define IMAGE_SYM_CLASS_WEAK_EXTERNAL 0x0069
+
+#define IMAGE_SYM_CLASS_CLR_TOKEN 0x006B
+
+// type packing constants
+
+#define N_BTMASK 0x000F
+#define N_TMASK  0x0030
+#define N_TMASK1 0x00C0
+#define N_TMASK2 0x00F0
+#define N_BTSHFT 4
+#define N_TSHIFT 2
+// MACROS
+
+// Basic Type of  x
+#define BTYPE(x) ((x) &N_BTMASK)
+
+// Is x a pointer?
+#ifndef ISPTR
+#define ISPTR(x) (((x) &N_TMASK) == (IMAGE_SYM_DTYPE_POINTER << N_BTSHFT))
+#endif
+
+// Is x a function?
+#ifndef ISFCN
+#define ISFCN(x) (((x) &N_TMASK) == (IMAGE_SYM_DTYPE_FUNCTION << N_BTSHFT))
+#endif
+
+// Is x an array?
+
+#ifndef ISARY
+#define ISARY(x) (((x) &N_TMASK) == (IMAGE_SYM_DTYPE_ARRAY << N_BTSHFT))
+#endif
+
+// Is x a structure, union, or enumeration TAG?
+#ifndef ISTAG
+#define ISTAG(x)                                                            \
+  ((x) == IMAGE_SYM_CLASS_STRUCT_TAG || (x) == IMAGE_SYM_CLASS_UNION_TAG || \
+   (x) == IMAGE_SYM_CLASS_ENUM_TAG)
+#endif
+
+#ifndef INCREF
+#define INCREF(x)                                                            \
+  ((((x) & ~N_BTMASK) << N_TSHIFT) | (IMAGE_SYM_DTYPE_POINTER << N_BTSHFT) | \
+   ((x) &N_BTMASK))
+#endif
+#ifndef DECREF
+#define DECREF(x) ((((x) >> N_TSHIFT) & ~N_BTMASK) | ((x) &N_BTMASK))
+#endif
+
 #pragma pack(pop)
 
-#endif  // _WIN32
+#endif  // _WIN32 || defined(__CYGWIN__)
 
 #define CVINFO_PDB70_CVSIGNATURE 0x53445352  // "RSDS"
 #define CVINFO_PDB20_CVSIGNATURE 0x3031424e  // "NB10"
+#define CODEVIEW_SIGNATURE_MTOC  0x434f544d  // "MTOC"
+
+#pragma pack(push, 1)
 
 typedef struct _CV_HEADER
 {
@@ -520,6 +793,13 @@ typedef struct _CV_INFO_PDB70
   BYTE PdbFileName[1];
 } CV_INFO_PDB70, *PCV_INFO_PDB70;
 
+typedef struct _MTOC_ENTRY
+{
+  DWORD Signature;
+  BYTE uuid[16];
+  BYTE PdbFileName[1];
+} MTOC_ENTRY, *PMTOC_ENTRY;
+
 typedef struct _VERSION_INFO
 {
   WORD Length;
@@ -557,6 +837,23 @@ typedef struct _WIN_CERTIFICATE
 
 #define RICH_VERSION_ID(id_version)      (id_version >> 16)
 #define RICH_VERSION_VERSION(id_version) (id_version & 0xFFFF)
+#define IMAGE_DEBUG_TYPE_UNKNOWN         0
+#define IMAGE_DEBUG_TYPE_COFF            1
+#define IMAGE_DEBUG_TYPE_CODEVIEW        2
+#define IMAGE_DEBUG_TYPE_FPO             3
+#define IMAGE_DEBUG_TYPE_MISC            4
+#define IMAGE_DEBUG_TYPE_EXCEPTION       5
+#define IMAGE_DEBUG_TYPE_FIXUP           6
+#define IMAGE_DEBUG_TYPE_OMAP_TO_SRC     7
+#define IMAGE_DEBUG_TYPE_OMAP_FROM_SRC   8
+#define IMAGE_DEBUG_TYPE_BORLAND         9
+#define IMAGE_DEBUG_TYPE_RESERVED10      10
+#define IMAGE_DEBUG_TYPE_CLSID           11
+#define IMAGE_DEBUG_TYPE_VC_FEATURE      12
+#define IMAGE_DEBUG_TYPE_POGO            13
+#define IMAGE_DEBUG_TYPE_ILTCG           14
+#define IMAGE_DEBUG_TYPE_MPX             15
+#define IMAGE_DEBUG_TYPE_REPRO           16
 
 typedef struct _RICH_VERSION_INFO
 {
@@ -577,5 +874,9 @@ typedef struct _RICH_SIGNATURE
 #define RICH_DANS 0x536e6144  // "DanS"
 #define RICH_RICH 0x68636952  // "Rich"
 
+#define PE_PAGE_SIZE   0x1000
+#define PE_SECTOR_SIZE 0x0200
+
 #pragma pack(pop)
+
 #endif
