@@ -42,6 +42,7 @@ typedef struct _IMPORT_FUNCTION
   char* name;
   uint8_t has_ordinal;
   uint16_t ordinal;
+  uint64_t rva;
 
   struct _IMPORT_FUNCTION* next;
 
@@ -83,9 +84,4 @@ int64_t pe_rva_to_offset(PE* pe, uint64_t rva);
 
 char* ord_lookup(char* dll, uint16_t ord);
 
-#if HAVE_LIBCRYPTO
-#include <openssl/asn1.h>
-time_t ASN1_get_time_t(const ASN1_TIME* time);
-#endif
-
 #endif
