@@ -177,7 +177,8 @@ int yr_parser_emit_with_arg_reloc(
 
 int yr_parser_emit_pushes_for_strings(
     yyscan_t yyscanner,
-    const char* identifier)
+    const char* identifier,
+    int* count)
 {
   YR_COMPILER* compiler = yyget_extra(yyscanner);
 
@@ -215,11 +216,17 @@ int yr_parser_emit_pushes_for_strings(
 
         string->flags |= STRING_FLAGS_REFERENCED;
         string->flags &= ~STRING_FLAGS_FIXED_OFFSET;
+        string->flags &= ~STRING_FLAGS_SINGLE_MATCH;
         matching++;
       }
     }
   }
 
+  if (count != NULL)
+  {
+    *count = matching;
+  }
+
   if (matching == 0)
   {
     yr_compiler_set_error_extra_info(
@@ -231,7 +238,10 @@ int yr_parser_emit_pushes_for_strings(
 
 // Emit OP_PUSH_RULE instructions for all rules whose identifier has given
 // prefix.
-int yr_parser_emit_pushes_for_rules(yyscan_t yyscanner, const char* prefix)
+int yr_parser_emit_pushes_for_rules(
+    yyscan_t yyscanner,
+    const char* prefix,
+    int* count)
 {
   YR_COMPILER* compiler = yyget_extra(yyscanner);
 
@@ -278,6 +288,11 @@ int yr_parser_emit_pushes_for_rules(yyscan_t yyscanner, const char* prefix)
     rule++;
   }
 
+  if (count != NULL)
+  {
+    *count = matching;
+  }
+
   if (matching == 0)
   {
     yr_compiler_set_error_extra_info(compiler, prefix);
@@ -289,39 +304,42 @@ int yr_parser_emit_pushes_for_rules(yyscan_t yyscanner, const char* prefix)
 
 int yr_parser_emit_push_const(yyscan_t yyscanner, uint64_t argument)
 {
-  uint64_t u = (uint64_t) argument;
-  uint8_t buf[9];
-  int bufsz = 1;
-  if (u == YR_UNDEFINED)
+  uint8_t opcode[9];
+  int opcode_len = 1;
+
+  if (argument == YR_UNDEFINED)
   {
-    buf[0] = OP_PUSH_U;
+    opcode[0] = OP_PUSH_U;
   }
-  else if (u <= 0xff)
+  else if (argument <= 0xff)
   {
-    buf[0] = OP_PUSH_8;
-    bufsz += sizeof(uint8_t);
-    buf[1] = (uint8_t) argument;
+    opcode[0] = OP_PUSH_8;
+    opcode[1] = (uint8_t) argument;
+    opcode_len += sizeof(uint8_t);
   }
-  else if (u <= 0xffff)
+  else if (argument <= 0xffff)
   {
-    buf[0] = OP_PUSH_16;
-    bufsz += sizeof(uint16_t);
-    *((uint16_t*) (buf + 1)) = (uint16_t) argument;
+    opcode[0] = OP_PUSH_16;
+    uint16_t u = (uint16_t) argument;
+    memcpy(opcode + 1, &u, sizeof(uint16_t));
+    opcode_len += sizeof(uint16_t);
   }
-  else if (u <= 0xffffffff)
+  else if (argument <= 0xffffffff)
   {
-    buf[0] = OP_PUSH_32;
-    bufsz += sizeof(uint32_t);
-    *((uint32_t*) (buf + 1)) = (uint32_t) argument;
+    opcode[0] = OP_PUSH_32;
+    uint32_t u = (uint32_t) argument;
+    memcpy(opcode + 1, &u, sizeof(uint32_t));
+    opcode_len += sizeof(uint32_t);
   }
   else
   {
-    buf[0] = OP_PUSH;
-    bufsz += sizeof(uint64_t);
-    *((uint64_t*) (buf + 1)) = (uint64_t) argument;
+    opcode[0] = OP_PUSH;
+    memcpy(opcode + 1, &argument, sizeof(uint64_t));
+    opcode_len += sizeof(uint64_t);
   }
+
   return yr_arena_write_data(
-      yyget_extra(yyscanner)->arena, YR_CODE_SECTION, buf, bufsz, NULL);
+      yyget_extra(yyscanner)->arena, YR_CODE_SECTION, opcode, opcode_len, NULL);
 }
 
 int yr_parser_check_types(
@@ -467,35 +485,17 @@ static int _yr_parser_write_string(
     literal_string = yr_re_ast_extract_literal(re_ast);
 
     if (literal_string != NULL)
-    {
-      modifier.flags |= STRING_FLAGS_LITERAL;
       free_literal = true;
-    }
-    else
-    {
-      // Non-literal strings can't be marked as fixed offset because once we
-      // find a string atom in the scanned data we don't know the offset where
-      // the string should start, as the non-literal strings can contain
-      // variable-length portions.
-
-      modifier.flags &= ~STRING_FLAGS_FIXED_OFFSET;
-    }
   }
   else
   {
     literal_string = str;
-    modifier.flags |= STRING_FLAGS_LITERAL;
   }
 
-  string->flags = modifier.flags;
-  string->rule_idx = compiler->current_rule_idx;
-  string->idx = compiler->current_string_idx;
-  string->fixed_offset = YR_UNDEFINED;
-  string->chained_to = NULL;
-  string->string = NULL;
-
-  if (modifier.flags & STRING_FLAGS_LITERAL)
+  if (literal_string != NULL)
   {
+    modifier.flags |= STRING_FLAGS_LITERAL;
+
     result = _yr_compiler_store_data(
         compiler,
         literal_string->c_string,
@@ -518,6 +518,12 @@ static int _yr_parser_write_string(
   }
   else
   {
+    // Non-literal strings can't be marked as fixed offset because once we
+    // find a string atom in the scanned data we don't know the offset where
+    // the string should start, as the non-literal strings can contain
+    // variable-length portions.
+    modifier.flags &= ~STRING_FLAGS_FIXED_OFFSET;
+
     // Emit forwards code
     result = yr_re_ast_emit_code(re_ast, compiler->arena, false);
 
@@ -534,6 +540,11 @@ static int _yr_parser_write_string(
           min_atom_quality);
   }
 
+  string->flags = modifier.flags;
+  string->rule_idx = compiler->current_rule_idx;
+  string->idx = compiler->current_string_idx;
+  string->fixed_offset = YR_UNDEFINED;
+
   if (result == ERROR_SUCCESS)
   {
     // Add the string to Aho-Corasick automaton.
@@ -690,7 +701,6 @@ int yr_parser_reduce_string_declaration(
     modifier.flags |= STRING_FLAGS_DOT_ALL;
 
   if (!(modifier.flags & STRING_FLAGS_WIDE) &&
-      !(modifier.flags & STRING_FLAGS_XOR) &&
       !(modifier.flags & STRING_FLAGS_BASE64 ||
         modifier.flags & STRING_FLAGS_BASE64_WIDE))
   {
@@ -738,23 +748,37 @@ int yr_parser_reduce_string_declaration(
     if (modifier.flags & STRING_FLAGS_HEXADECIMAL)
       result = yr_re_parse_hex(str->c_string, &re_ast, &re_error);
     else if (modifier.flags & STRING_FLAGS_REGEXP)
-      result = yr_re_parse(str->c_string, &re_ast, &re_error);
+    {
+      int flags = RE_PARSER_FLAG_NONE;
+      if (compiler->strict_escape)
+        flags |= RE_PARSER_FLAG_ENABLE_STRICT_ESCAPE_SEQUENCES;
+      result = yr_re_parse(str->c_string, &re_ast, &re_error, flags);
+    }
     else
       result = yr_base64_ast_from_string(str, modifier, &re_ast, &re_error);
 
     if (result != ERROR_SUCCESS)
     {
-      snprintf(
-          message,
-          sizeof(message),
-          "invalid %s \"%s\": %s",
-          (modifier.flags & STRING_FLAGS_HEXADECIMAL) ? "hex string"
-                                                      : "regular expression",
-          identifier,
-          re_error.message);
+      if (result == ERROR_UNKNOWN_ESCAPE_SEQUENCE)
+      {
+        yywarning(
+          yyscanner,
+          "unknown escape sequence");
+      }
+      else 
+      {
+        snprintf(
+            message,
+            sizeof(message),
+            "invalid %s \"%s\": %s",
+            (modifier.flags & STRING_FLAGS_HEXADECIMAL) ? "hex string"
+                                                        : "regular expression",
+            identifier,
+            re_error.message);
 
-      yr_compiler_set_error_extra_info(compiler, message);
-      goto _exit;
+        yr_compiler_set_error_extra_info(compiler, message);
+        goto _exit;
+      }
     }
 
     if (re_ast->flags & RE_FLAGS_FAST_REGEXP)
@@ -763,11 +787,11 @@ int yr_parser_reduce_string_declaration(
     if (re_ast->flags & RE_FLAGS_GREEDY)
       modifier.flags |= STRING_FLAGS_GREEDY_REGEXP;
 
-    // Regular expressions in the strings section can't mix greedy and ungreedy
-    // quantifiers like .* and .*?. That's because these regular expressions can
-    // be matched forwards and/or backwards depending on the atom found, and we
-    // need the regexp to be all-greedy or all-ungreedy to be able to properly
-    // calculate the length of the match.
+    // Regular expressions in the strings section can't mix greedy and
+    // ungreedy quantifiers like .* and .*?. That's because these regular
+    // expressions can be matched forwards and/or backwards depending on the
+    // atom found, and we need the regexp to be all-greedy or all-ungreedy to
+    // be able to properly calculate the length of the match.
 
     if ((re_ast->flags & RE_FLAGS_GREEDY) &&
         (re_ast->flags & RE_FLAGS_UNGREEDY))
@@ -832,14 +856,15 @@ int yr_parser_reduce_string_declaration(
 
       if (YR_ARENA_IS_NULL_REF(*string_ref))
       {
-        // This is the first string in the chain, the string reference returned
-        // by this function must point to this string.
+        // This is the first string in the chain, the string reference
+        // returned by this function must point to this string.
         *string_ref = ref;
       }
       else
       {
-        // This is not the first string in the chain, set the appropriate flags
-        // and fill the chained_to, chain_gap_min and chain_gap_max fields.
+        // This is not the first string in the chain, set the appropriate
+        // flags and fill the chained_to, chain_gap_min and chain_gap_max
+        // fields.
         YR_STRING* prev_string = (YR_STRING*) yr_arena_get_ptr(
             compiler->arena,
             YR_STRINGS_TABLE,
@@ -856,14 +881,14 @@ int yr_parser_reduce_string_declaration(
         // head of the string chain can have a fixed offset.
         new_string->flags &= ~STRING_FLAGS_FIXED_OFFSET;
 
-        // There is a previous string, but that string wasn't marked as part of
-        // a chain because we can't do that until knowing there will be another
-        // string, let's flag it now the we know.
+        // There is a previous string, but that string wasn't marked as part
+        // of a chain because we can't do that until knowing there will be
+        // another string, let's flag it now the we know.
         prev_string->flags |= STRING_FLAGS_CHAIN_PART;
 
         // There is a previous string, so this string is part of a chain, but
-        // there will be no more strings because there are no more AST to split,
-        // which means that this is the chain's tail.
+        // there will be no more strings because there are no more AST to
+        // split, which means that this is the chain's tail.
         if (remainder_re_ast == NULL)
           new_string->flags |= STRING_FLAGS_CHAIN_PART |
                                STRING_FLAGS_CHAIN_TAIL;
@@ -993,12 +1018,12 @@ int yr_parser_reduce_rule_declaration_phase_1(
   compiler->current_rule_idx = compiler->next_rule_idx;
   compiler->next_rule_idx++;
 
-  // The OP_INIT_RULE instruction behaves like a jump. When the rule is disabled
-  // it skips over the rule's code and go straight to the next rule's code. The
-  // jmp_offset_ref variable points to the jump's offset. The offset is set to 0
-  // as we don't know the jump target yet. When we finish generating the rule's
-  // code in yr_parser_reduce_rule_declaration_phase_2 the jump offset is set to
-  // its final value.
+  // The OP_INIT_RULE instruction behaves like a jump. When the rule is
+  // disabled it skips over the rule's code and go straight to the next rule's
+  // code. The jmp_offset_ref variable points to the jump's offset. The offset
+  // is set to 0 as we don't know the jump target yet. When we finish
+  // generating the rule's code in yr_parser_reduce_rule_declaration_phase_2
+  // the jump offset is set to its final value.
 
   FAIL_ON_ERROR(yr_parser_emit_with_arg_int32(
       yyscanner, OP_INIT_RULE, 0, NULL, &jmp_offset_ref));
@@ -1060,13 +1085,26 @@ int yr_parser_reduce_rule_declaration_phase_2(
     // Only the heading fragment in a chain of strings (the one with
     // chained_to == NULL) must be referenced. All other fragments
     // are never marked as referenced.
+    //
+    // Any string identifier that starts with '_' can be unreferenced. Anonymous
+    // strings must always be referenced.
 
-    if (!STRING_IS_REFERENCED(string) && string->chained_to == NULL)
+    if (!STRING_IS_REFERENCED(string) && string->chained_to == NULL &&
+        (STRING_IS_ANONYMOUS(string) ||
+         (!STRING_IS_ANONYMOUS(string) && string->identifier[1] != '_')))
     {
       yr_compiler_set_error_extra_info(
           compiler, string->identifier) return ERROR_UNREFERENCED_STRING;
     }
 
+    // If a string is unreferenced we need to unset the FIXED_OFFSET flag so
+    // that it will match anywhere.
+    if (!STRING_IS_REFERENCED(string) && string->chained_to == NULL &&
+        STRING_IS_FIXED_OFFSET(string))
+    {
+      string->flags &= ~STRING_FLAGS_FIXED_OFFSET;
+    }
+
     strings_in_rule++;
 
     if (strings_in_rule > max_strings_per_rule)
@@ -1088,7 +1126,7 @@ int yr_parser_reduce_rule_declaration_phase_2(
                            compiler->arena, YR_CODE_SECTION) -
                        fixup->ref.offset + 1;
 
-  *jmp_offset_addr = jmp_offset;
+  memcpy(jmp_offset_addr, &jmp_offset, sizeof(jmp_offset));
 
   // Remove fixup from the stack.
   compiler->fixup_stack_head = fixup->next;
@@ -1110,7 +1148,7 @@ int yr_parser_reduce_string_identifier(
   YR_STRING* string;
   YR_COMPILER* compiler = yyget_extra(yyscanner);
 
-  if (strcmp(identifier, "$") == 0)  // is an anonymous string ?
+  if (strcmp(identifier, "$") == 0)            // is an anonymous string ?
   {
     if (compiler->loop_for_of_var_index >= 0)  // inside a loop ?
     {
