@@ -47,7 +47,9 @@ order to avoid confusion with operating system threads.
 #include <yara/mem.h>
 #include <yara/re.h>
 #include <yara/re_lexer.h>
+#include <yara/strutils.h>
 #include <yara/threading.h>
+#include <yara/unaligned.h>
 #include <yara/utils.h>
 
 #define EMIT_BACKWARDS               0x01
@@ -60,6 +62,13 @@ order to avoid confusion with operating system threads.
 
 typedef uint8_t RE_SPLIT_ID_TYPE;
 
+// RE_REPEAT_ARGS and RE_REPEAT_ANY_ARGS are structures that are embedded in
+// the regexp's instruction stream. As such, they are not always aligned to
+// 8-byte boundaries, so they need to be "packed" structures in order to prevent
+// issues due to unaligned memory accesses.
+#pragma pack(push)
+#pragma pack(1)
+
 typedef struct _RE_REPEAT_ARGS
 {
   uint16_t min;
@@ -75,6 +84,8 @@ typedef struct _RE_REPEAT_ANY_ARGS
 
 } RE_REPEAT_ANY_ARGS;
 
+#pragma pack(pop)
+
 typedef struct _RE_EMIT_CONTEXT
 {
   YR_ARENA* arena;
@@ -203,9 +214,13 @@ void yr_re_ast_destroy(RE_AST* re_ast)
 // Parses a regexp but don't emit its code. A further call to
 // yr_re_ast_emit_code is required to get the code.
 //
-int yr_re_parse(const char* re_string, RE_AST** re_ast, RE_ERROR* error)
+int yr_re_parse(
+    const char* re_string,
+    RE_AST** re_ast,
+    RE_ERROR* error,
+    int flags)
 {
-  return yr_parse_re_string(re_string, re_ast, error);
+  return yr_parse_re_string(re_string, re_ast, error, flags);
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -224,14 +239,18 @@ int yr_re_parse_hex(const char* hex_string, RE_AST** re_ast, RE_ERROR* error)
 int yr_re_compile(
     const char* re_string,
     int flags,
+    int parser_flags,
     YR_ARENA* arena,
     YR_ARENA_REF* ref,
     RE_ERROR* error)
 {
   RE_AST* re_ast;
   RE _re;
+  int result;
 
-  FAIL_ON_ERROR(yr_re_parse(re_string, &re_ast, error));
+  result = yr_re_parse(re_string, &re_ast, error, parser_flags);
+  if (result != ERROR_UNKNOWN_ESCAPE_SEQUENCE)
+    FAIL_ON_ERROR(result);
 
   _re.flags = flags;
 
@@ -244,7 +263,7 @@ int yr_re_compile(
 
   yr_re_ast_destroy(re_ast);
 
-  return ERROR_SUCCESS;
+  return result;
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -371,7 +390,7 @@ int _yr_re_node_has_unbounded_quantifier_for_dot(RE_NODE* re_node)
 ////////////////////////////////////////////////////////////////////////////////
 // Detects the use of .*, .+ or .{x,} in a regexp. The use of wildcards with
 // quantifiers that don't have a reasonably small upper bound causes a
-// performance penalty. This function dectects such cases in order to warn the
+// performance penalty. This function detects such cases in order to warn the
 // user about this.
 //
 int yr_re_ast_has_unbounded_quantifier_for_dot(RE_AST* re_ast)
@@ -674,16 +693,13 @@ int _yr_emit_split(
   return ERROR_SUCCESS;
 }
 
-#define current_re_code_offset() \
-  yr_arena_get_current_offset(emit_context->arena, YR_RE_CODE_SECTION)
-
 static int _yr_re_emit(
     RE_EMIT_CONTEXT* emit_context,
     RE_NODE* re_node,
     int flags,
     YR_ARENA_REF* code_ref)
 {
-  yr_arena_off_t jmp_offset;
+  int16_t jmp_offset;
 
   yr_arena_off_t bookmark_1 = 0;
   yr_arena_off_t bookmark_2 = 0;
@@ -721,6 +737,15 @@ static int _yr_re_emit(
         NULL));
     break;
 
+  case RE_NODE_NOT_LITERAL:
+    FAIL_ON_ERROR(_yr_emit_inst_arg_uint8(
+        emit_context,
+        RE_OPCODE_NOT_LITERAL,
+        re_node->value,
+        &instruction_ref,
+        NULL));
+    break;
+
   case RE_NODE_MASKED_LITERAL:
     FAIL_ON_ERROR(_yr_emit_inst_arg_uint16(
         emit_context,
@@ -730,6 +755,15 @@ static int _yr_re_emit(
         NULL));
     break;
 
+  case RE_NODE_MASKED_NOT_LITERAL:
+    FAIL_ON_ERROR(_yr_emit_inst_arg_uint16(
+        emit_context,
+        RE_OPCODE_MASKED_NOT_LITERAL,
+        re_node->mask << 8 | re_node->value,
+        &instruction_ref,
+        NULL));
+    break;
+
   case RE_NODE_WORD_CHAR:
     FAIL_ON_ERROR(
         _yr_emit_inst(emit_context, RE_OPCODE_WORD_CHAR, &instruction_ref));
@@ -828,15 +862,18 @@ static int _yr_re_emit(
     FAIL_ON_ERROR(_yr_re_emit(
         emit_context, re_node->children_head, flags, &instruction_ref));
 
-    jmp_offset = instruction_ref.offset - current_re_code_offset();
+    bookmark_1 = yr_arena_get_current_offset(
+        emit_context->arena, YR_RE_CODE_SECTION);
 
-    if (jmp_offset < INT16_MIN)
+    if (instruction_ref.offset - bookmark_1 < INT16_MIN)
       return ERROR_REGULAR_EXPRESSION_TOO_LARGE;
 
+    jmp_offset = (int16_t) (instruction_ref.offset - bookmark_1);
+
     FAIL_ON_ERROR(_yr_emit_split(
         emit_context,
         re_node->greedy ? RE_OPCODE_SPLIT_B : RE_OPCODE_SPLIT_A,
-        (int16_t) jmp_offset,
+        jmp_offset,
         NULL,
         NULL));
 
@@ -859,26 +896,32 @@ static int _yr_re_emit(
     FAIL_ON_ERROR(
         _yr_re_emit(emit_context, re_node->children_head, flags, NULL));
 
-    jmp_offset = instruction_ref.offset - current_re_code_offset();
+    bookmark_1 = yr_arena_get_current_offset(
+        emit_context->arena, YR_RE_CODE_SECTION);
 
-    if (jmp_offset < INT16_MIN)
+    if (instruction_ref.offset - bookmark_1 < INT16_MIN)
       return ERROR_REGULAR_EXPRESSION_TOO_LARGE;
 
+    jmp_offset = (int16_t) (instruction_ref.offset - bookmark_1);
+
     // Emit jump with offset set to 0.
 
     FAIL_ON_ERROR(_yr_emit_inst_arg_int16(
-        emit_context, RE_OPCODE_JUMP, (int16_t) jmp_offset, NULL, NULL));
+        emit_context, RE_OPCODE_JUMP, jmp_offset, NULL, NULL));
 
-    jmp_offset = current_re_code_offset() - instruction_ref.offset;
+    bookmark_1 = yr_arena_get_current_offset(
+        emit_context->arena, YR_RE_CODE_SECTION);
 
-    if (jmp_offset > INT16_MAX)
+    if (bookmark_1 - instruction_ref.offset > INT16_MAX)
       return ERROR_REGULAR_EXPRESSION_TOO_LARGE;
 
+    jmp_offset = (int16_t) (bookmark_1 - instruction_ref.offset);
+
     // Update split offset.
     split_offset_addr = (int16_t*) yr_arena_ref_to_ptr(
         emit_context->arena, &split_offset_ref);
 
-    *split_offset_addr = (int16_t) jmp_offset;
+    memcpy(split_offset_addr, &jmp_offset, sizeof(jmp_offset));
     break;
 
   case RE_NODE_ALT:
@@ -913,30 +956,36 @@ static int _yr_re_emit(
         &jmp_instruction_ref,
         &jmp_offset_ref));
 
-    jmp_offset = current_re_code_offset() - instruction_ref.offset;
+    bookmark_1 = yr_arena_get_current_offset(
+        emit_context->arena, YR_RE_CODE_SECTION);
 
-    if (jmp_offset > INT16_MAX)
+    if (bookmark_1 - instruction_ref.offset > INT16_MAX)
       return ERROR_REGULAR_EXPRESSION_TOO_LARGE;
 
+    jmp_offset = (int16_t) (bookmark_1 - instruction_ref.offset);
+
     // Update split offset.
     split_offset_addr = (int16_t*) yr_arena_ref_to_ptr(
         emit_context->arena, &split_offset_ref);
 
-    *split_offset_addr = (int16_t) jmp_offset;
+    memcpy(split_offset_addr, &jmp_offset, sizeof(jmp_offset));
 
     FAIL_ON_ERROR(
         _yr_re_emit(emit_context, re_node->children_tail, flags, NULL));
 
-    jmp_offset = current_re_code_offset() - jmp_instruction_ref.offset;
+    bookmark_1 = yr_arena_get_current_offset(
+        emit_context->arena, YR_RE_CODE_SECTION);
 
-    if (jmp_offset > INT16_MAX)
+    if (bookmark_1 - jmp_instruction_ref.offset > INT16_MAX)
       return ERROR_REGULAR_EXPRESSION_TOO_LARGE;
 
+    jmp_offset = (int16_t) (bookmark_1 - jmp_instruction_ref.offset);
+
     // Update offset for jmp instruction.
     jmp_offset_addr = (int16_t*) yr_arena_ref_to_ptr(
         emit_context->arena, &jmp_offset_ref);
 
-    *jmp_offset_addr = (int16_t) jmp_offset;
+    memcpy(jmp_offset_addr, &jmp_offset, sizeof(jmp_offset));
     break;
 
   case RE_NODE_RANGE_ANY:
@@ -1037,7 +1086,8 @@ static int _yr_re_emit(
 
       repeat_args.offset = 0;
 
-      bookmark_1 = current_re_code_offset();
+      bookmark_1 = yr_arena_get_current_offset(
+          emit_context->arena, YR_RE_CODE_SECTION);
 
       FAIL_ON_ERROR(_yr_emit_inst_arg_struct(
           emit_context,
@@ -1048,7 +1098,8 @@ static int _yr_re_emit(
           emit_prolog ? NULL : &instruction_ref,
           &repeat_start_args_ref));
 
-      bookmark_2 = current_re_code_offset();
+      bookmark_2 = yr_arena_get_current_offset(
+          emit_context->arena, YR_RE_CODE_SECTION);
 
       FAIL_ON_ERROR(_yr_re_emit(
           emit_context,
@@ -1056,12 +1107,13 @@ static int _yr_re_emit(
           flags | EMIT_DONT_SET_FORWARDS_CODE | EMIT_DONT_SET_BACKWARDS_CODE,
           NULL));
 
-      bookmark_3 = current_re_code_offset();
+      bookmark_3 = yr_arena_get_current_offset(
+          emit_context->arena, YR_RE_CODE_SECTION);
 
       if (bookmark_2 - bookmark_3 < INT32_MIN)
         return ERROR_REGULAR_EXPRESSION_TOO_LARGE;
 
-      repeat_args.offset = (int32_t)(bookmark_2 - bookmark_3);
+      repeat_args.offset = (int32_t) (bookmark_2 - bookmark_3);
 
       FAIL_ON_ERROR(_yr_emit_inst_arg_struct(
           emit_context,
@@ -1072,7 +1124,8 @@ static int _yr_re_emit(
           NULL,
           NULL));
 
-      bookmark_4 = current_re_code_offset();
+      bookmark_4 = yr_arena_get_current_offset(
+          emit_context->arena, YR_RE_CODE_SECTION);
 
       repeat_start_args_addr = (RE_REPEAT_ARGS*) yr_arena_ref_to_ptr(
           emit_context->arena, &repeat_start_args_ref);
@@ -1080,12 +1133,13 @@ static int _yr_re_emit(
       if (bookmark_4 - bookmark_1 > INT32_MAX)
         return ERROR_REGULAR_EXPRESSION_TOO_LARGE;
 
-      repeat_start_args_addr->offset = (int32_t)(bookmark_4 - bookmark_1);
+      repeat_start_args_addr->offset = (int32_t) (bookmark_4 - bookmark_1);
     }
 
     if (emit_split)
     {
-      bookmark_1 = current_re_code_offset();
+      bookmark_1 = yr_arena_get_current_offset(
+          emit_context->arena, YR_RE_CODE_SECTION);
 
       FAIL_ON_ERROR(_yr_emit_split(
           emit_context,
@@ -1106,7 +1160,8 @@ static int _yr_re_emit(
 
     if (emit_split)
     {
-      bookmark_2 = current_re_code_offset();
+      bookmark_2 = yr_arena_get_current_offset(
+          emit_context->arena, YR_RE_CODE_SECTION);
 
       if (bookmark_2 - bookmark_1 > INT16_MAX)
         return ERROR_REGULAR_EXPRESSION_TOO_LARGE;
@@ -1114,7 +1169,9 @@ static int _yr_re_emit(
       split_offset_addr = (int16_t*) yr_arena_ref_to_ptr(
           emit_context->arena, &split_offset_ref);
 
-      *split_offset_addr = (int16_t)(bookmark_2 - bookmark_1);
+      jmp_offset = (int16_t) (bookmark_2 - bookmark_1);
+
+      memcpy(split_offset_addr, &jmp_offset, sizeof(jmp_offset));
     }
 
     break;
@@ -1419,6 +1476,7 @@ static int _yr_re_fiber_sync(
 
   while (fiber != last)
   {
+    int16_t jmp_len;
     uint8_t opcode = *fiber->ip;
 
     switch (opcode)
@@ -1461,12 +1519,10 @@ static int _yr_re_fiber_sync(
         // Branch A continues at the next instruction
         branch_a->ip += (sizeof(RE_SPLIT_ID_TYPE) + 3);
 
-        // Branch B adds the offset encoded in the opcode to its instruction
-        // pointer.
-        branch_b->ip += *(int16_t*)(
-              branch_b->ip
-              + 1  // opcode size
-              + sizeof(RE_SPLIT_ID_TYPE));
+        // Branch B adds the offset indicated by the split instruction.
+        jmp_len = yr_unaligned_i16(branch_b->ip + 1 + sizeof(RE_SPLIT_ID_TYPE));
+
+        branch_b->ip += jmp_len;
 
 #ifdef YR_PARANOID_MODE
         // In normal conditions this should never happen. But with compiled
@@ -1593,7 +1649,8 @@ static int _yr_re_fiber_sync(
       break;
 
     case RE_OPCODE_JUMP:
-      fiber->ip += *(int16_t*) (fiber->ip + 1);
+      jmp_len = yr_unaligned_i16(fiber->ip + 1);
+      fiber->ip += jmp_len;
       break;
 
     default:
@@ -1655,6 +1712,7 @@ int yr_re_exec(
   const uint8_t* input;
   const uint8_t* ip;
 
+  uint16_t opcode_args;
   uint8_t mask;
   uint8_t value;
   uint8_t character_size;
@@ -1671,6 +1729,9 @@ int yr_re_exec(
   int kill;
   int action;
 
+  bool prev_is_word_char = false;
+  bool input_is_word_char = false;
+
 #define ACTION_NONE      0
 #define ACTION_CONTINUE  1
 #define ACTION_KILL      2
@@ -1699,12 +1760,14 @@ int yr_re_exec(
 
   if (flags & RE_FLAGS_BACKWARDS)
   {
+    // Signedness conversion is sound as long as YR_RE_SCAN_LIMIT <= INT_MAX
     max_bytes_matched = (int) yr_min(input_backwards_size, YR_RE_SCAN_LIMIT);
     input -= character_size;
     input_incr = -input_incr;
   }
   else
   {
+    // Signedness conversion is sound as long as YR_RE_SCAN_LIMIT <= INT_MAX
     max_bytes_matched = (int) yr_min(input_forwards_size, YR_RE_SCAN_LIMIT);
   }
 
@@ -1778,10 +1841,23 @@ int yr_re_exec(
         fiber->ip += 2;
         break;
 
+      case RE_OPCODE_NOT_LITERAL:
+        prolog;
+
+        // We don't need to take into account the case-insensitive
+        // case because this opcode is only used with hex strings,
+        // which can't be case-insensitive.
+
+        match = (*input != *(ip + 1));
+        action = match ? ACTION_NONE : ACTION_KILL;
+        fiber->ip += 2;
+        break;
+
       case RE_OPCODE_MASKED_LITERAL:
         prolog;
-        value = *(int16_t*) (ip + 1) & 0xFF;
-        mask = *(int16_t*) (ip + 1) >> 8;
+        opcode_args = yr_unaligned_u16(ip + 1);
+        mask = opcode_args >> 8;
+        value = opcode_args & 0xFF;
 
         // We don't need to take into account the case-insensitive
         // case because this opcode is only used with hex strings,
@@ -1792,6 +1868,21 @@ int yr_re_exec(
         fiber->ip += 3;
         break;
 
+      case RE_OPCODE_MASKED_NOT_LITERAL:
+        prolog;
+        opcode_args = yr_unaligned_u16(ip + 1);
+        mask = opcode_args >> 8;
+        value = opcode_args & 0xFF;
+
+        // We don't need to take into account the case-insensitive
+        // case because this opcode is only used with hex strings,
+        // which can't be case-insensitive.
+
+        match = ((*input & mask) != value);
+        action = match ? ACTION_NONE : ACTION_KILL;
+        fiber->ip += 3;
+        break;
+
       case RE_OPCODE_CLASS:
         prolog;
         match = _yr_re_is_char_in_class(
@@ -1856,27 +1947,30 @@ int yr_re_exec(
 
       case RE_OPCODE_WORD_BOUNDARY:
       case RE_OPCODE_NON_WORD_BOUNDARY:
-
-        if (bytes_matched == 0 && input_backwards_size < character_size)
+        if (input - input_incr + character_size <= input_data + input_forwards_size &&
+            input - input_incr >= input_data - input_backwards_size)
         {
-          match = true;
-        }
-        else if (bytes_matched >= max_bytes_matched)
-        {
-          match = true;
+          prev_is_word_char = _yr_re_is_word_char(
+              input - input_incr, character_size);
         }
         else
         {
-          assert(input < input_data + input_forwards_size);
-          assert(input >= input_data - input_backwards_size);
-
-          assert(input - input_incr < input_data + input_forwards_size);
-          assert(input - input_incr >= input_data - input_backwards_size);
-
-          match = _yr_re_is_word_char(input, character_size) !=
-                  _yr_re_is_word_char(input - input_incr, character_size);
+          prev_is_word_char = false;
         }
 
+        if (input + character_size <= input_data + input_forwards_size &&
+            input >= input_data - input_backwards_size)
+        {
+          input_is_word_char = _yr_re_is_word_char(input, character_size);
+        }
+        else
+        {
+          input_is_word_char = false;
+        }
+
+        match = (prev_is_word_char && !input_is_word_char) ||
+                (!prev_is_word_char && input_is_word_char);
+
         if (*ip == RE_OPCODE_NON_WORD_BOUNDARY)
           match = !match;
 
@@ -1988,14 +2082,84 @@ int yr_re_exec(
 }
 
 ////////////////////////////////////////////////////////////////////////////////
-// This function replaces yr_re_exec for regular expressions marked with flag
-// RE_FLAGS_FAST_REGEXP. These are regular expression whose code contain only
-// the following operations: RE_OPCODE_LITERAL, RE_OPCODE_MASKED_LITERAL,
-// RE_OPCODE_ANY, RE_OPCODE_REPEAT_ANY_UNGREEDY and RE_OPCODE_MATCH. Some
-// examples of regular expressions that can be executed with this function are:
+// Helper function that creates a RE_FAST_EXEC_POSITION by either allocating it
+// or reusing a previously allocated one from a pool.
 //
-//  /foobar/
-//  /foo.*?bar/
+static int _yr_re_fast_exec_position_create(
+    RE_FAST_EXEC_POSITION_POOL* pool,
+    RE_FAST_EXEC_POSITION** new_position)
+{
+  RE_FAST_EXEC_POSITION* position;
+
+  if (pool->head != NULL)
+  {
+    position = pool->head;
+    pool->head = position->next;
+  }
+  else
+  {
+    position = (RE_FAST_EXEC_POSITION*) yr_malloc(
+        sizeof(RE_FAST_EXEC_POSITION));
+
+    if (position == NULL)
+      return ERROR_INSUFFICIENT_MEMORY;
+  }
+
+  position->input = NULL;
+  position->round = 0;
+  position->next = NULL;
+  position->prev = NULL;
+
+  *new_position = position;
+
+  return ERROR_SUCCESS;
+}
+
+////////////////////////////////////////////////////////////////////////////////
+// Helper function that moves a list of RE_FAST_EXEC_POSITION structures to a
+// pool represented by a RE_FAST_EXEC_POSITION_POOL. Receives pointers to the
+// pool, the first item, and last item in the list.
+//
+static void _yr_re_fast_exec_destroy_position_list(
+    RE_FAST_EXEC_POSITION_POOL* pool,
+    RE_FAST_EXEC_POSITION* first,
+    RE_FAST_EXEC_POSITION* last)
+{
+  last->next = pool->head;
+
+  if (pool->head != NULL)
+    pool->head->prev = last;
+
+  pool->head = first;
+}
+
+////////////////////////////////////////////////////////////////////////////////
+// This function replaces yr_re_exec for regular expressions marked with flag
+// RE_FLAGS_FAST_REGEXP. These regular expressions are derived from hex strings
+// that don't contain alternatives, like for example:
+//
+//   { 01 02 03 04 [0-2] 04 05 06 07 }
+//
+// The regexp's code produced by such strings can be matched by a faster, less
+// general algorithm, and it contains only the following opcodes:
+//
+//   * RE_OPCODE_LITERAL
+//   * RE_OPCODE_MASKED_LITERAL,
+//   * RE_OPCODE_NOT_LITERAL
+//   * RE_OPCODE_MASKED_NOT_LITERAL
+//   * RE_OPCODE_ANY
+//   * RE_OPCODE_REPEAT_ANY_UNGREEDY
+//   * RE_OPCODE_MATCH.
+//
+// The regexp's code is executed one instruction at time, and the function
+// maintains a list of positions within the input for tracking different
+// matching alternatives, these positions are described by an instance of
+// RE_FAST_EXEC_POSITION. With each instruction the list of positions is
+// updated, removing those where the input data doesn't match the current
+// instruction, or adding new positions if the instruction is
+// RE_OPCODE_REPEAT_ANY_UNGREEDY. The list of positions is maintained sorted
+// by the value of the pointer they hold to the input, and it doesn't contain
+// duplicated pointer values.
 //
 int yr_re_fast_exec(
     YR_SCAN_CONTEXT* context,
@@ -2010,162 +2174,317 @@ int yr_re_fast_exec(
 {
   RE_REPEAT_ANY_ARGS* repeat_any_args;
 
-  const uint8_t* code_stack[YR_MAX_FAST_RE_STACK];
-  const uint8_t* input_stack[YR_MAX_FAST_RE_STACK];
-  int matches_stack[YR_MAX_FAST_RE_STACK];
-
-  const uint8_t* input = input_data;
-  const uint8_t* next_input;
-  const uint8_t* ip = code;
-  const uint8_t* next_opcode;
-
-  uint8_t mask;
-  uint8_t value;
-
-  int stop;
-  int input_incr;
-  int sp = 0;
+  // Pointers to the first and last position in the list.
+  RE_FAST_EXEC_POSITION* first;
+  RE_FAST_EXEC_POSITION* last;
 
+  int input_incr = flags & RE_FLAGS_BACKWARDS ? -1 : 1;
   int bytes_matched;
   int max_bytes_matched;
 
-  max_bytes_matched = flags & RE_FLAGS_BACKWARDS ? (int) input_backwards_size
-                                                 : (int) input_forwards_size;
+  if (flags & RE_FLAGS_BACKWARDS)
+    max_bytes_matched = (int) yr_min(input_backwards_size, YR_RE_SCAN_LIMIT);
+  else
+    max_bytes_matched = (int) yr_min(input_forwards_size, YR_RE_SCAN_LIMIT);
 
-  input_incr = flags & RE_FLAGS_BACKWARDS ? -1 : 1;
+  const uint8_t* ip = code;
+
+  // Create the first position in the list, which points to the start of the
+  // input data. Intially this is the only position, more positions will be
+  // created every time RE_OPCODE_REPEAT_ANY_UNGREEDY is executed.
+  FAIL_ON_ERROR(_yr_re_fast_exec_position_create(
+      &context->re_fast_exec_position_pool, &first));
+
+  first->round = 0;
+  first->input = input_data;
+  first->prev = NULL;
+  first->next = NULL;
 
   if (flags & RE_FLAGS_BACKWARDS)
-    input--;
+    first->input--;
 
-  code_stack[sp] = code;
-  input_stack[sp] = input;
-  matches_stack[sp] = 0;
-  sp++;
+  // As we are starting with a single position, the last one and the first one
+  // are the same.
+  last = first;
 
-  while (sp > 0)
+  // Round is incremented with every regxp instruction.
+  int round = 0;
+
+  // Keep in the loop until the list of positions gets empty. Positions are
+  // removed from the list when they don't match the current instruction in
+  // the code.
+  while (first != NULL)
   {
-    sp--;
-    ip = code_stack[sp];
-    input = input_stack[sp];
-    bytes_matched = matches_stack[sp];
-    stop = false;
+    RE_FAST_EXEC_POSITION* current = first;
 
-    while (!stop)
+    // Iterate over all the positions, executing the current instruction at
+    // all of them.
+    while (current != NULL)
     {
-      if (*ip == RE_OPCODE_MATCH)
+      RE_FAST_EXEC_POSITION* next = current->next;
+
+      // Ignore any position in the list whose round number is different from
+      // the current round. This prevents new positions added to the list in
+      // this round from being taken into account in this same round.
+      if (current->round != round)
       {
+        current = next;
+        continue;
+      }
+
+      bytes_matched = flags & RE_FLAGS_BACKWARDS
+                          ? (int) (input_data - current->input - 1)
+                          : (int) (current->input - input_data);
+
+      uint16_t opcode_args;
+      uint8_t mask;
+      uint8_t value;
+
+      bool match = false;
+
+      switch (*ip)
+      {
+      case RE_OPCODE_ANY:
+        if (bytes_matched >= max_bytes_matched)
+          break;
+
+        match = true;
+        current->input += input_incr;
+        break;
+
+      case RE_OPCODE_LITERAL:
+        if (bytes_matched >= max_bytes_matched)
+          break;
+
+        if (*current->input == *(ip + 1))
+        {
+          match = true;
+          current->input += input_incr;
+        }
+        break;
+
+      case RE_OPCODE_NOT_LITERAL:
+        if (bytes_matched >= max_bytes_matched)
+          break;
+
+        if (*current->input != *(ip + 1))
+        {
+          match = true;
+          current->input += input_incr;
+        }
+        break;
+
+      case RE_OPCODE_MASKED_LITERAL:
+        if (bytes_matched >= max_bytes_matched)
+          break;
+
+        opcode_args = yr_unaligned_u16(ip + 1);
+        mask = opcode_args >> 8;
+        value = opcode_args & 0xFF;
+
+        if ((*current->input & mask) == value)
+        {
+          match = true;
+          current->input += input_incr;
+        }
+        break;
+
+      case RE_OPCODE_MASKED_NOT_LITERAL:
+        if (bytes_matched >= max_bytes_matched)
+          break;
+
+        opcode_args = yr_unaligned_u16(ip + 1);
+        mask = opcode_args >> 8;
+        value = opcode_args & 0xFF;
+
+        if ((*current->input & mask) != value)
+        {
+          match = true;
+          current->input += input_incr;
+        }
+        break;
+
+      case RE_OPCODE_REPEAT_ANY_UNGREEDY:
+        repeat_any_args = (RE_REPEAT_ANY_ARGS*) (ip + 1);
+
+        if (bytes_matched + repeat_any_args->min >= max_bytes_matched)
+          break;
+
+        match = true;
+
+        const uint8_t* next_opcode = ip + 1 + sizeof(RE_REPEAT_ANY_ARGS);
+
+        // insertion_point indicates the item in the list of inputs after which
+        // the newly created inputs will be inserted.
+        RE_FAST_EXEC_POSITION* insertion_point = current;
+
+        for (int j = repeat_any_args->min + 1; j <= repeat_any_args->max; j++)
+        {
+          // bytes_matched is the number of bytes matched so far, and j is the
+          // minimum number of bytes that are still pending to match. If these
+          // two numbers sum more than the maximum number of bytes that can be
+          // matched the loop can be aborted. The position that we were about
+          // to create can't lead to a match without exceeding
+          // max_bytes_matched.
+          if (bytes_matched + j >= max_bytes_matched)
+            break;
+
+          const uint8_t* next_input = current->input + j * input_incr;
+
+          // Find the point where next_input should be inserted. The list is
+          // kept sorted by pointer in increasing order, the insertion point is
+          // an item that has a pointer lower or equal than next_input, but
+          // whose next item have a pointer that is larger.
+          while (insertion_point->next != NULL &&
+                 next_input >= insertion_point->next->input)
+          {
+            insertion_point = insertion_point->next;
+          }
+
+          // If the pointer at the insertion point is equal to next_input we
+          // don't need to insert next_input in the list as this input already
+          // exists.
+          if (next_input == insertion_point->input)
+            continue;
+
+          // The next opcode is RE_OPCODE_LITERAL, but the literal doesn't
+          // match with the byte at next_input, we don't need to add this
+          // input to the list as we already know that it won't match in the
+          // next round and will be deleted from the list anyways.
+          if (*(next_opcode) == RE_OPCODE_LITERAL &&
+              *(next_opcode + 1) != *next_input)
+            continue;
+
+          RE_FAST_EXEC_POSITION* new_input;
+
+          FAIL_ON_ERROR_WITH_CLEANUP(
+              _yr_re_fast_exec_position_create(
+                  &context->re_fast_exec_position_pool, &new_input),
+              // Cleanup
+              _yr_re_fast_exec_destroy_position_list(
+                  &context->re_fast_exec_position_pool, first, last));
+
+          new_input->input = next_input;
+          new_input->round = round + 1;
+          new_input->prev = insertion_point;
+          new_input->next = insertion_point->next;
+          insertion_point->next = new_input;
+
+          if (new_input->next != NULL)
+            new_input->next->prev = new_input;
+
+          if (insertion_point == last)
+            last = new_input;
+        }
+
+        current->input += input_incr * repeat_any_args->min;
+        break;
+
+      case RE_OPCODE_MATCH:
+
         if (flags & RE_FLAGS_EXHAUSTIVE)
         {
-          FAIL_ON_ERROR(callback(
-              // When matching forwards the matching data always starts at
-              // input_data, when matching backwards it starts at input + 1 or
-              // input_data - input_backwards_size if input + 1 is outside the
-              // buffer.
-              flags & RE_FLAGS_BACKWARDS
-                  ? yr_max(input + 1, input_data - input_backwards_size)
-                  : input_data,
-              // The number of matched bytes should not be larger than
-              // max_bytes_matched.
-              yr_min(bytes_matched, max_bytes_matched),
-              flags,
-              callback_args));
-
-          break;
+          FAIL_ON_ERROR_WITH_CLEANUP(
+              callback(
+                  // When matching forwards the matching data always starts at
+                  // input_data, when matching backwards it starts at input + 1
+                  // or input_data - input_backwards_size if input + 1 is
+                  // outside the buffer.
+                  flags & RE_FLAGS_BACKWARDS
+                      ? yr_max(
+                            current->input + 1,
+                            input_data - input_backwards_size)
+                      : input_data,
+                  // The number of matched bytes should not be larger than
+                  // max_bytes_matched.
+                  yr_min(bytes_matched, max_bytes_matched),
+                  flags,
+                  callback_args),
+              // Cleanup
+              _yr_re_fast_exec_destroy_position_list(
+                  &context->re_fast_exec_position_pool, first, last));
         }
         else
         {
           if (matches != NULL)
             *matches = bytes_matched;
 
+          _yr_re_fast_exec_destroy_position_list(
+              &context->re_fast_exec_position_pool, first, last);
+
           return ERROR_SUCCESS;
         }
-      }
-
-      if (bytes_matched >= max_bytes_matched)
-        break;
-
-      switch (*ip)
-      {
-      case RE_OPCODE_LITERAL:
-
-        if (*input == *(ip + 1))
-        {
-          bytes_matched++;
-          input += input_incr;
-          ip += 2;
-        }
-        else
-        {
-          stop = true;
-        }
-
-        break;
-
-      case RE_OPCODE_MASKED_LITERAL:
-
-        value = *(int16_t*) (ip + 1) & 0xFF;
-        mask = *(int16_t*) (ip + 1) >> 8;
-
-        if ((*input & mask) == value)
-        {
-          bytes_matched++;
-          input += input_incr;
-          ip += 3;
-        }
-        else
-        {
-          stop = true;
-        }
-
-        break;
-
-      case RE_OPCODE_ANY:
-
-        bytes_matched++;
-        input += input_incr;
-        ip += 1;
-
-        break;
-
-      case RE_OPCODE_REPEAT_ANY_UNGREEDY:
-
-        repeat_any_args = (RE_REPEAT_ANY_ARGS*) (ip + 1);
-        next_opcode = ip + 1 + sizeof(RE_REPEAT_ANY_ARGS);
-
-        for (int i = repeat_any_args->min + 1; i <= repeat_any_args->max; i++)
-        {
-          if (bytes_matched + i >= max_bytes_matched)
-            break;
-
-          next_input = input + i * input_incr;
-
-          if (*(next_opcode) != RE_OPCODE_LITERAL ||
-              (*(next_opcode) == RE_OPCODE_LITERAL &&
-               *(next_opcode + 1) == *next_input))
-          {
-            if (sp >= YR_MAX_FAST_RE_STACK)
-              return ERROR_TOO_MANY_RE_FIBERS;
-
-            code_stack[sp] = next_opcode;
-            input_stack[sp] = next_input;
-            matches_stack[sp] = bytes_matched + i;
-            sp++;
-          }
-        }
-
-        input += input_incr * repeat_any_args->min;
-        bytes_matched += repeat_any_args->min;
-
-        ip = next_opcode;
-
         break;
 
       default:
+        printf("non-supported opcode: %d\n", *ip);
         assert(false);
       }
+
+      if (match)
+      {
+        current->round = round + 1;
+      }
+      else
+      {
+        // Remove current from the list. If the item being removed is the first
+        // one or the last one, the first and last pointers should be updated.
+        // The removed item is put into the pool for later reuse.
+
+        if (current == first)
+          first = current->next;
+
+        if (current == last)
+          last = current->prev;
+
+        if (current->prev != NULL)
+          current->prev->next = current->next;
+
+        if (current->next != NULL)
+          current->next->prev = current->prev;
+
+        current->prev = NULL;
+        current->next = context->re_fast_exec_position_pool.head;
+
+        if (current->next != NULL)
+          current->next->prev = current;
+
+        context->re_fast_exec_position_pool.head = current;
+      }
+
+      current = next;
+
+    }  // while (current != NULL)
+
+    // Move instruction pointer (ip) to next instruction. Each opcode has
+    // a different size.
+    switch (*ip)
+    {
+    case RE_OPCODE_ANY:
+      ip += 1;
+      break;
+    case RE_OPCODE_LITERAL:
+    case RE_OPCODE_NOT_LITERAL:
+      ip += 2;
+      break;
+    case RE_OPCODE_MASKED_LITERAL:
+    case RE_OPCODE_MASKED_NOT_LITERAL:
+      ip += 3;
+      break;
+    case RE_OPCODE_REPEAT_ANY_UNGREEDY:
+      ip += 1 + sizeof(RE_REPEAT_ANY_ARGS);
+      break;
+    case RE_OPCODE_MATCH:
+      break;
+    default:
+      assert(false);
     }
+
+    round++;
   }
 
+  // If this point is reached no matches were found.
+
   if (matches != NULL)
     *matches = -1;
 
@@ -2220,6 +2539,10 @@ static void _yr_re_print_node(RE_NODE* re_node, uint32_t indent)
     printf("Lit(%c)", re_node->value);
     break;
 
+  case RE_NODE_NOT_LITERAL:
+    printf("NotLit(%c)", re_node->value);
+    break;
+
   case RE_NODE_MASKED_LITERAL:
     printf("MaskedLit(%02X,%02X)", re_node->value, re_node->mask);
     break;
@@ -2266,6 +2589,30 @@ static void _yr_re_print_node(RE_NODE* re_node, uint32_t indent)
     printf(")");
     break;
 
+  case RE_NODE_EMPTY:
+    printf("Empty");
+    break;
+
+  case RE_NODE_ANCHOR_START:
+    printf("AnchorStart");
+    break;
+
+  case RE_NODE_ANCHOR_END:
+    printf("AnchorEnd");
+    break;
+
+  case RE_NODE_WORD_BOUNDARY:
+    printf("WordBoundary");
+    break;
+
+  case RE_NODE_NON_WORD_BOUNDARY:
+    printf("NonWordBoundary");
+    break;
+
+  case RE_NODE_RANGE_ANY:
+    printf("RangeAny");
+    break;
+
   default:
     printf("???");
     break;
