@@ -1,6 +1,5 @@
-#include <stdint.h>
 #include <stddef.h>
-
+#include <stdint.h>
 #include <yara.h>
 
 
@@ -26,25 +25,23 @@ extern "C" int LLVMFuzzerInitialize(int* argc, char*** argv)
 }
 
 
-int callback(YR_SCAN_CONTEXT* context, int message, void* message_data, void* user_data)
+int callback(
+    YR_SCAN_CONTEXT* context,
+    int message,
+    void* message_data,
+    void* user_data)
 {
   return CALLBACK_CONTINUE;
 }
 
 
-extern "C" int LLVMFuzzerTestOneInput(const uint8_t *data, size_t size)
+extern "C" int LLVMFuzzerTestOneInput(const uint8_t* data, size_t size)
 {
   if (rules == NULL)
     return 0;
 
   yr_rules_scan_mem(
-      rules,
-      data,
-      size,
-      SCAN_FLAGS_NO_TRYCATCH,
-      callback,
-      NULL,
-      0);
+      rules, data, size, SCAN_FLAGS_NO_TRYCATCH, callback, NULL, 0);
 
   return 0;
 }
