@@ -71,6 +71,38 @@ int main(int argc, char** argv)
       }",
       "AABAAB");
 
+  assert_true_rule_blob(
+      "import \"math\" \
+      rule test { \
+        condition: \
+          math.count(0x41, 4, 10) == 1  \
+      }",
+      "AABAAB");
+
+  assert_true_rule_blob(
+      "import \"math\" \
+      rule test { \
+        condition: \
+          not defined math.count(0x41, 10, 3)  \
+      }",
+      "AABAAB");
+
+  assert_true_rule_blob(
+      "import \"math\" \
+      rule test { \
+        condition: \
+          not defined math.count(0x41, 0, -3)  \
+      }",
+      "AABAAB");
+
+  assert_true_rule_blob(
+      "import \"math\" \
+      rule test { \
+        condition: \
+          not defined math.count(-1)  \
+      }",
+      "AABAAB");
+
   assert_true_rule_blob(
       "import \"math\" \
       rule test { \
@@ -85,7 +117,7 @@ int main(int argc, char** argv)
         condition: \
           math.percentage(0x41) > 0.39 and math.percentage(0x41) < 0.41 \
       }",
-      "ABAB"); // Blob matching includes terminating zero byte
+      "ABAB");  // Blob matching includes terminating zero byte
 
   assert_true_rule_blob(
       "import \"math\" \
@@ -95,6 +127,38 @@ int main(int argc, char** argv)
       }",
       "ABABCDEF");
 
+  assert_true_rule_blob(
+      "import \"math\" \
+      rule test { \
+        condition: \
+          not defined math.percentage(0x41, 0, 0) \
+      }",
+      "AABAAB");
+
+  assert_true_rule_blob(
+      "import \"math\" \
+      rule test { \
+        condition: \
+          not defined math.percentage(0x41, 10, 3)  \
+      }",
+      "AABAAB");
+
+  assert_true_rule_blob(
+      "import \"math\" \
+      rule test { \
+        condition: \
+          not defined math.percentage(0x41, 0, -3)  \
+      }",
+      "AABAAB");
+
+  assert_true_rule_blob(
+      "import \"math\" \
+      rule test { \
+        condition: \
+          not defined math.percentage(-1)  \
+      }",
+      "AABAAB");
+
   assert_true_rule_blob(
       "import \"math\" \
       rule test { \
@@ -103,6 +167,14 @@ int main(int argc, char** argv)
       }",
       "ABABA");
 
+  assert_true_rule_blob(
+      "import \"math\" \
+      rule test { \
+        condition: \
+          math.mode() == 0x41 \
+      }",
+      "ABAB");
+
   assert_true_rule_blob(
       "import \"math\" \
       rule test { \
